## -*- coding: utf-8 -*-
"""Helper for funittest scripts.
"""

import os
import sys
import inspect

from optparse import OptionParser
from funittest.utils import normalizeClassName, retrieveVersion
from funittest.problems import ModelOfProductImportError
from funittest.problems import OptionParserError
from funittest.core.discovery import PresetDiscovery, ProductDiscovery
from funittest.core.configuration import loadConfiguration, retrieveConfiguration

from funittest import interpreter
from funittest.log import logger

class BaseCommand(object):
    """Base for all funittest commands.
    
    Supports choosing:
    - a specific preset configuraton file,
    - a specific product library from which to load models,
    - dry run mode for running the command without executing it on a
      live site,
    - pause after each session, so you can inspect the results while
      running,
    - verbosity to know more about what is currently done,
    - the browser to use,
    - the url to use.
    """

    usage = "usage: %prog [options]"

    def __init__(self):
        loadConfiguration('funittest.cfg')

        self.preset_discovery = PresetDiscovery()
        self.product_discovery = ProductDiscovery()

    def error(self, msg):
        """error(msg : string)

        Print a usage message incorporating 'msg' to stderr and exit.
        If you override this in a subclass, it should not return -- it
        should either exit or raise an exception.
        """
        logger.log_usage(self.usage)
        logger.log_optionparser_error("funittest", msg)
        raise OptionParserError(msg)

    def customOptions(self, parser):
        """Add custom options to the script.
        """
    
    def parseOptions(self, extra=None):
        """Parse options of the script. 
    
        - usage is the usage of the script;
        - description of what does the script;
        - customOptions is a function used to add specific options to
          the parser.
        """
        interpreter.setPort("4444")
        interpreter.setHost("localhost")

        self.options, args = self._parseOptions(extra)
        if args:
            self.error("Arguments are not accepted. Use named options.")

        self.handle_pause_option()
        self.handle_dry_option()
        self.handle_browser_option()
        self.handle_preset_option()
        
        self.afterParseOptions()

    def _parseOptions(self, extra=None):
        parser = OptionParser(usage=self.usage,
                              description=self.description,
                              version=retrieveVersion())

        parser.add_option("--preset", 
                          dest="preset", 
                          default=None,
                          help="The preset configuration file from which to import settings (default: %default)")

        parser.add_option("-p", 
                          "--product", 
                          dest="product",
                          default=None,
                          help="The product library from which to load the tests (default: %default)")

        parser.add_option("--url", 
                          dest="browser_url",
                          help="The browser URL")

        parser.add_option("--browser", 
                          dest="browser_start_command",
                          help='The command string used to launch the browser, e.g. "firefox",  "iexplore",  "chrome", "safari" or "konqueror"')

        self.customOptions(parser)
        if extra:
            extra(parser)

        parser.add_option("--dry",
                          action="store_true",
                          dest="dry",
                          help="Run in dry mode without doing action for real")
        parser.add_option("--pause",
                          dest = "pause",
                          action="store_true",
                          help="Pause after each session, waiting for a key press")
        parser.add_option("-v",
                          "--verbose",
                          action="store_true",
                          dest="verbose",
                          help="Verbose mode describing every action made")

        return parser.parse_args()

    def handle_pause_option(self):
        if self.options.pause:
            interpreter.set_pause_on_ok(True)
            interpreter.set_pause_on_ko(True)
            interpreter.set_pause_on_interpreter_error(True)

    def handle_dry_option(self):
        if self.options.dry or not self.options.browser_url:
            interpreter.set_connection_state("OFFLINE")
            assert(interpreter.is_dry_run()==True)
        else:
            interpreter.set_connection_state("ONLINE")
            assert(interpreter.is_dry_run()==False)

    def handle_preset_option(self):
        if self.options.preset is None:
             return            
        if not self.options.preset in self.preset_discovery.available():
            message = 'Unknown preset "%s"' % self.options.preset
            message += '\nAvailable preset(s): %s' % ', '.join(self.preset_discovery.available())
            self.error(message)
        logger.start_preset(self.options.preset)
        version = self.preset_discovery.version(self.options.preset)
        version()
        preset = self.preset_discovery.preset(self.options.preset)
        self.dataproviders = preset()
        logger.end_preset()

    def handle_browser_option(self):
        browser_config = dict()
        if self.options.browser_url:
            browser_config['url'] = self.options.browser_url
        if self.options.browser_start_command:
            browser_config['type'] = self.options.browser_start_command
        config = retrieveConfiguration()
        config.set_configuration('browser', browser_config)

    def afterParseOptions(self):
        """Called after parsing options.
        """
        pass

    def lookupProduct(self, registry, product_name):
        """Lookup a product, or display an error.
        """
        if product_name is None:
            return None
        product = None
        try:
            product = getattr(registry, product_name)
        except ModelOfProductImportError:
            message = 'Product "%s" can\'t be found.' % product_name
            available = registry.get_products_info()
            if available:
                message += "\nLoaded product(s): "
                message += ', '.join(p['name'] for p in available)
            available = self.product_discovery.available()
            if available:
                message += "\nAvailable product(s): "
                message += ', '.join(available)
            self.error(message)
        return product

    def lookupModels(self, product, name, default=None):
        """Lookup a model through a product, or display an error.
        """
        models = []
        if name:
            try:
                models = [getattr(product, name),]
            except AttributeError:
                message = 'Product "%s" does not have a model called "%s"' % (self.options.product, name)
                available = product.get_models_info()
                if available:
                    message += "\nAvailable model(s):\n"
                    message += ', '.join(p['name'] for p in available)
                self.error(message)
        elif default is None and not product is None:
            models = product.get_models()
        else:
            if default is None:
                models = []
            else:
                models = default
        return models

    def run(self):
        """Running the command.
        
        This is designed to be overriden in subclass.
        """
        


class ModelBasedCommand(BaseCommand):
    """Model based command.
    
       Used for running scenario and test models. Allows selection of
       an extension to run, and fixing and varying variables to be
       used.
       
       Also allows listing available models and displaying their
       documentation.
    """

    stream = sys.stdout

    def customOptions(self, parser):
        """Basic options for commands using models.
        """

        parser.add_option("-e",
                          "--extension",
                          dest="extension",
                          help="Extension to use")
        parser.add_option("--fix",
                          dest="fix",
                          action="append",
                          help="Data to fix")
        parser.add_option("--vary",
                          dest="vary",
                          action="append",
                          help="Data to vary, vary information in format key:value")
        parser.add_option("--list",
                          dest = "list",
                          action="store_true",
                          help="List all models")
        parser.add_option("--doc",
                          dest = "doc",
                          action="store_true",
                          help="Print short documentation of models")
        parser.add_option("--source",
                          dest = "source",
                          action="store_true",
                          help="Print source code of models")

    def optionToList(self, option):
        """Convert an option to a list.
        """
        if option:
            return [o for o in option]
        return []

    def optionToDict(self, option, name):
        """Convert an option to a dict.
        """
    
        def validate(item):
            if not ':' in item:
                self.error('Wrong format for "%s" parameter. Must be key:value.' % name)
            return item.split(":")

        if option:
            return dict([validate(i) for i in option])
        return dict()

    def afterParseOptions(self):
        """Set fix and vary options.
        """
        BaseCommand.afterParseOptions(self)
        self.vary = self.optionToList(self.options.vary)
        self.fix = self.optionToDict(self.options.fix, "fix")

    def displayDocumentation(self, product, models):
        """Eventually display information about models, and return
        true if something has been done.
        """

        def model_name(model):
            return normalizeClassName(model)

        def model_method(model):
            name = model_name(model)
            self.stream.write("- %s(%s)\n" % (name,', '.join(model.schema.keys())))
            self.stream.flush()

        def model_list(doc, source):
            if models:
                self.stream.write("Model list:\n")
                for model in models:
                    if doc:
                        model_doc(model)
                    elif source:
                        model_source(model)
                    else:
                        model_method(model)
            else:
                self.stream.write("No models available.\n")

        def model_doc(model):
            model_method(model)
            self.stream.write('  ' + model.__doc__.replace('\n', '\n  ') + '\n')

        def model_source(model):
            model_method(model)
            module = model.__module
            if inspect.ismodule(module):
                file = inspect.getsourcefile(module)
                self.stream.write('  source: ' + file + '\n')
                lines, number = inspect.getsourcelines(getattr(module, model.__class__.__name__))
                self.stream.writelines(lines)                

        if self.options.list or self.options.doc or self.options.source:
            model_list(self.options.doc, self.options.source)
            return True
        return False
