from os.path import dirname, basename, join, abspath
import os, sys
import time

from funittest.log import _getLogPath
from funittest.log import logger
from funittest.log import InternalFilter
from funittest.log import NothingFilter
from funittest import interpreter
from funittest.problems import OptionParserError
from funittest.utils import SubProcess
from funittest.seleniumrc import DEFAULTTIMEOUT

class CommandRunner(object):
    """Run a command.
    """

    def __init__(self, cmd):
        self.cmd = cmd()

    def parseOptions(self, parser):
        """Add extra option for the parser.
        """
        pass

    def run(self):
        """Called after optionsParse, to call the following method in this
        order:

        0. parseOptions
        1. beforeRun
        2. run
        3. afterRun
        """

        try:
            self.cmd.parseOptions(self.parseOptions)
        except OptionParserError:
            sys.exit(1)

        try:
            self.beforeRun()
            status = self.cmd.run()
        except KeyboardInterrupt:
            status = 0
            pass                # Keyboard interrupt, just do nothing.
        except:
            self.afterRun()     # Error, do post run before to
                                # continue.
            raise
        self.afterRun()         # No error or Keyboard interrupt, do post run.
        return status

    def beforeRun(self):
        """Called before running the command.
        """

    def afterRun(self):
        """Called after running the command.
        """

class UnixCommandRunner(CommandRunner):
    """Cool command runner under Unix.
    """

    def __init__(self, cmd):
        self.set_sleep_seconds(10)

        super(UnixCommandRunner, self).__init__(cmd)

        script = sys.argv[0]
        bin_path = dirname(abspath(script))
        funittest_path = dirname(bin_path)

        console_logger = self.get_console_logger()
        if console_logger:
             if console_logger.filters:
                 pass # Never override any filter that is already there.
                      # Important for unit testing, where a filter is used
                      # that filters out all messages before they are sent
                      # to the console.
             else:
                 if "-v" in sys.argv:
                     # If verbosity is switched on, a filter is established that
                     # let's through any non-empty message.
                     console_logger.addFilter(NothingFilter(console_logger))
                 else:
                     # If verbosity is switched off, a filter is established
                     # that only let's relevant messages through, filtering
                     # out any internal messages.
                     console_logger.addFilter(InternalFilter(console_logger))

        self.selenium = join(bin_path, 'seleniumrc')
        if not os.access(self.selenium, os.X_OK):
            logger.log_seleniumrc_not_found(self.selenium)
            self.selenium = False

        self.instance = join(bin_path, 'instance')
        if not os.access(self.instance, os.X_OK):
            self.instance = False
        
        self.subprocess = SubProcess()
                
    def get_console_logger(self):
        return logger.retrieveLogger("console")

    def parseOptions(self, parser):
        super(UnixCommandRunner, self).parseOptions(parser)

        parser.add_option("--xvfb",
                          dest="xvfb",
                          action="store_true",
                          help="Start Selenium RC in a virtual framebuffer")

    def runCommandInBackground(self, command, args=[], env=None):
        log_name = _getLogPath(name=basename(command) + '.log')
        env = env and env or os.environ
        error_callback = self.cmd.error
        verbose = self.cmd.options.verbose
        self.subprocess.start(log_name, command, args, env, error_callback, verbose)

    def stopCommandsInBackground(self):
        self.subprocess.stop_all()

    def beforeRun(self):
        options = self.cmd.options
        if not hasattr(options, 'doc'):
            options.doc = False
        if not hasattr(options, 'list'):
            options.list = False
        if not (options.dry or options.doc or options.list):
            if self.selenium:
                selenium_env = os.environ.copy()
                if options.xvfb:
                    selenium_display = os.environ.get('FUNITTEST_DISPLAY', ':4')
                    self.runCommandInBackground('Xvfb', [selenium_display, '-ac'])
                    selenium_env['DISPLAY'] = selenium_display
                self.runCommandInBackground(self.selenium, ['-timeout', str(DEFAULTTIMEOUT)], env=selenium_env)
            if self.instance:
                self.runCommandInBackground(self.instance, ['fg',])
            self.sleep()

    def set_sleep_seconds(self, seconds):
        self.sleep_seconds = seconds

    def get_sleep_seconds(self):
        return self.sleep_seconds

    def sleep(self):
        # let Zope and Selenium begin to start.
        time.sleep(self.get_sleep_seconds())

    def afterRun(self):
        options = self.cmd.options
        if not hasattr(options, 'doc'):
            options.doc = False
        if not hasattr(options, 'list'):
            options.list = False
        if not (options.dry or options.doc or options.list):
            if options.verbose:
                print "Cleaning background process ..."
            self.stopCommandsInBackground()

class WindowsCommandRunner(CommandRunner):
    """Cool command runner under Unix.
    """

    def __init__(self, cmd):
        self.cmd = cmd()
                        
    def get_console_logger(self):
        pass
        
    def parseOptions(self, parser):
        pass
        
    def runCommandInBackground(self, command, args=[], env=None):
        pass
        
    def stopCommandsInBackground(self):
        pass
        
    def beforeRun(self):
        pass
        
    def set_sleep_seconds(self, seconds):
        pass

    def get_sleep_seconds(self):
        pass
        
    def sleep(self):
        pass
        
    def afterRun(self):
        pass

def run_command(cmd, simple=False):
    """Run a command with the correct command runner.
    """
    
    if simple:
        runner = CommandRunner
    else:
        if sys.platform.startswith('win'):
            runner = WindowsCommandRunner
        else:
            runner = UnixCommandRunner
    r = runner(cmd)
    return r.run()

