import random

from funittest.log import logger
from funittest.problems import EnforceDictionaryArguments
from funittest.registry.verificationmanager import verification as state

import inspect
class decorator_base(object):
    # Taken over from http://pycurious.org/?p=54
    # Default arguments to use when decorator is invoked without arguments
    default_positional_args = ()

    def __init__(self, *args, **kwargs):
        if len(args) == 1 and not kwargs and inspect.isfunction(args[0]):
            # Case where only a single argument is given and it's a function.
            # We assume this means we're being called as a decorator without
            # any arguments.
            self._wrapped = args[0]
            args = self.default_positional_args
        else:
            self._wrapped = None

        self._arg_init(*args, **kwargs)

    def _arg_init(self, *args, **kwargs):
        """
        Initialization function called for both the args, and no args cases. 
        You may have to do some special handling for the defaults of 
        positional arguments since you're not going to get any when the 
        decorator is invoked without ().

        """
        self._args = args
        self._kwargs = kwargs

    def _shim(self, instance, *args, **kwargs):
        """
        All calls to a decorated functions pass through this method. All special
        argument inspection/interception should be done here.

        """
        return self._wrapped(instance, *args, **kwargs)

    def __call__(self, wrapped):
        """
        Handles the case where the decorator is given arguments. Stores the wrapped
        function and returns self so that the descriptor mechanism works.

        """
        self._wrapped = wrapped
        return self

    def __get__(self, instance, owner):
        """
        Behave like a descriptor so we can get our owner class instance.

        """
        def shim(*args, **kwargs):
            return self._shim(instance, *args, **kwargs)
        return shim

class verify(decorator_base):
    default_positional_args = ('Default Name',)

    def _arg_init(self, name, description="Default Description"):
        self._name = name
        self._description = description

    def _shim(self, instance, *args, **kwargs):
        method_name = self._wrapped.func_name

        # Only if verification is on, should the "on_" method be called
        if state.is_verification_done():
            # We need to inform the state model that the action is starting
            # That means the state can be catched before the logical functional
            # method is called can be saved
            state.actionStarting(instance, method_name, args)

        # Enforce use of dictionaries
        for argument in args:
            if type(argument) !=type({}):
                raise EnforceDictionaryArguments(instance, method_name, args, kwargs)

        logger.start_logical_method(instance, method_name, args, kwargs)

        result = self._wrapped(instance, *args, **kwargs)

        # Try to get the logical method as part of a composite
        # execution behavior
        comp_postfix = "_composite_execution_behaviour"

        # Call the execution behaviour method that was chosen
        ebm = getattr(instance, method_name + comp_postfix, None)

        if ebm:
            ebm_method = random.choice(ebm[:])
            
            logger.start_behaviour_method(ebm_method.func_name, args, kwargs)
            ebm_method(instance, *args, **kwargs)
            logger.end_behaviour_method(ebm_method.func_name, args, kwargs)

        if state.is_verification_done():
            # We need to call the verification manager when the
            # operation ends
            state.actualstate = state.snapshotCurrentState(instance, method_name)
            verified, info = state.actionEnding(instance, method_name, args)
            if not verified:
                state.verification_error(info)

        logger.end_logical_method(instance, method_name, args, kwargs)

        return result
