
import sys

import martian
from martian import ModuleGrokker
from martian import InstanceGrokker
from martian import ClassGrokker

from funittest.core.interfaces import IPolicyDataProvider
from funittest.core.interfaces import IDataProvider
from funittest.core.interfaces import ILogical
from funittest.core.interfaces import IExpectedStateGenerator
from funittest.core.interfaces import ITest
from funittest.core.interfaces import IPolicyTest
from funittest.core.interfaces import IParameterTest
from funittest.core.interfaces import IScript
from funittest.core.interfaces import IPhysical
from funittest.core.interfaces import IScenario

from funittest.core.discovery import ProductDiscovery
from funittest.utils import listAvailablePythonModules
from funittest.utils import normalizeClassName
from funittest.utils import normalizeName
from funittest.log import logger 

class DataProviderGrokker(InstanceGrokker):
    martian.component(object)
    def grok(self, name, obj, **kw):
        if IDataProvider.providedBy(obj):
            from funittest import dataprovider
            dataprovider.register_model(product_name=kw['product'], 
                                        model=obj, 
                                        name=normalizeClassName(obj),
                                        origin=kw['origin'],
                                        module=sys.modules.get(obj.__module__, None))
            return True
        return False

dataprovidergrokker = DataProviderGrokker()

class ModelGrokker(ClassGrokker):
    martian.component(object)
    def grok(self, name, obj, **kw):    
        if ILogical.implementedBy(obj):
            from funittest import logical
            instance = obj()
            logical.register_model(product_name=kw['product'], 
                                   model=instance, 
                                   name=normalizeName(obj),
                                   origin=kw['origin'],
                                   module=sys.modules.get(obj.__module__, None))
            return True
        if IExpectedStateGenerator.implementedBy(obj):
            from funittest import state
            instance = obj()
            state.register_model(product_name=kw['product'], 
                                 model=instance, 
                                 name=normalizeName(obj),
                                 origin=kw['origin'],
                                 module=sys.modules.get(obj.__module__, None))
            return True
        if ITest.implementedBy(obj) or IPolicyTest.implementedBy(obj) or IParameterTest.implementedBy(obj):
            from funittest import tests
            instance = obj()
            tests.register_model(product_name=kw['product'], 
                                 model=instance, 
                                 name=normalizeName(obj),
                                 origin=kw['origin'],
                                 module=sys.modules.get(obj.__module__, None))
            return True
        if IScript.implementedBy(obj):
            from funittest import scripts
            instance = obj()
            scripts.register_model(product_name=kw['product'], 
                                   model=instance, 
                                   name=normalizeName(obj),
                                   origin=kw['origin'],
                                   module=sys.modules.get(obj.__module__, None))
            return True
        if IPhysical.implementedBy(obj):
            from funittest import physical
            instance = obj()
            physical.register_model(product_name=kw['product'], 
                                    model=instance, 
                                    name=normalizeName(obj),
                                    origin=kw['origin'],
                                    module=sys.modules.get(obj.__module__, None))
            return True
        if IScenario.implementedBy(obj):
            from funittest import scenarios
            instance = obj()
            scenarios.register_model(product_name=kw['product'], 
                                     model=instance, 
                                     name=normalizeName(obj),
                                     origin=kw['origin'],
                                     module=sys.modules.get(obj.__module__, None))
            return True
        return False

modelgrokker = ModelGrokker()

def load_product_version(product, version="newest"):
    discovery = ProductDiscovery()
    if not (product in discovery.available()):
        raise ValueError, 'Unknown product "%s"' % product

    version = version.replace(".", "_")
    model_path = "funittest.lib.%s.%s"

    versions = discovery.versions(product).available()
    if not (version in versions):
        raise ValueError, 'Invalid product version "%s"' % version

    reset_product_registries(product)

    next_version = versions.pop()
    load_funittest_model(model_path  % (product, next_version))
    while version != next_version:
        next_version = versions.pop()
        logger.load_models_for_version(next_version)
        load_funittest_model(model_path  % (product, next_version))

def reset_product_registries(product):
    from funittest import physical
    from funittest import dataprovider
    from funittest import logical
    from funittest import scenarios
    from funittest import tests
    from funittest import scripts
    from funittest import state
    physical.reset_product_models(product)
    dataprovider.reset_product_models(product)
    logical.reset_product_models(product)
    scenarios.reset_product_models(product)
    tests.reset_product_models(product)
    scripts.reset_product_models(product)
    state.reset_product_models(product)

def load_funittest_model(module, physical='physical'):
    """
    Loads the module containing funittest models.

    By default, the pysical models are supposed to be written in 
    Selenium and can be loaded like this:

    load_funittest_model("funittest.lib.myfunittestlibrary.newest")

    The physical models are then at the following location:

    funittest.plone/funittest/lib/newest/physical

    If you want to use Twill instead of Selenium, you can load the
    models like this:

    load_funittest_model("funittest.lib.plone.newest", "twill")

    In this case the twill model is located here:

    funittest.plone/funittest/lib/plone/newest/twill
    """
    # Assemble the different possible model prefixes and types
    model_types = [("dp", "data"),
                    ("lfm", "logical"),
                    ("pom", physical),
                    ("esg", "state"),
                    ("tst", "tests"),
                    ("spt", "scripts"),
                    ("sce", "scenarios"),]

    for model_prefix, model_type in model_types:
        models = get_models_of_type(module, model_prefix, model_type)
        import_models_of_type(models)

def get_models_of_type(module, model_prefix, model_type):
    """
    Find all the models of a certain type in the given model type, doing
    a directory listing, and return a list of all file names without the file 
    extension that start with the model prefix and end with ".py".

    - A module could be "funittest.lib.plone.plone"
    - A model_prefix could be "dp"
    - A model_type could be "data"
    """

    all = listAvailablePythonModules('%s.%s' % (module, model_type))
    return [all[p] for p in all if p.startswith(model_prefix)]

def import_models_of_type(models):
    """
    Import all models known to exist in a model type module.
    
    - model can be for example funittest.lib.plone253.plone.data.dp_user
    """
    for model in models:
        __import__(model)
        product = model.split('.')[2]
        version = model.split('.')[3]

        modulegrokker = ModuleGrokker()
        modulegrokker.register(dataprovidergrokker)
        modulegrokker.grok(model,
                           sys.modules[model], 
                           origin = 'funittest.lib.%s.%s' % (product, version),
                           product = product)
        
        modulegrokker = ModuleGrokker()
        modulegrokker.register(modelgrokker)
        modulegrokker.grok(model,
                          sys.modules[model], 
                          origin = 'funittest.lib.%s.%s' % (product, version),
                          product = product)

