# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from zope.interface import Interface, Attribute

# Configuration

class IDiscovery(Interface):
    """Discover python modules in directory.
    """

    def available():
        """Return available modules for this discovery instance.
        """

class IConfiguration(Interface):
    """Funittest extended configuration.
    """

    def default():
        """Return default settings.
        """
    

# Logger interface

class ILogger(Interface):
    """Logger module.
    """

    def startTestLogging(ident):
        """Start the test identified by ident list.
        """

    def closeTestLogging():
        """Close specific logger for the test.
        """

    def retrieveLogger(name):
        """Return the logger called name if available.
        """

class ITestLogger(Interface):
    """Specific logger which are only used to log test (and not others
    things).
    """

    def start(ident):
        """Start the logger for the test identified by ident.
        """

    def close():
        """Close the logger (the test is finished).
        """
    def path():
        """Return the path/identifiant of the log.
        """

    def emit(record):
        """Log the entry record.
        """
    
# Model registery
    
class IModelProvider(Interface):
    """Model provider.
    """

    def get_models():
        """Return available models.
        """


class IModelDocumentationProvider(Interface):
    """Provide documentation about models.
    """

    def get_models_info():
        """Return information about models.
        """

class IProductProvider(IModelProvider):
    """Product provider.
    """

    def has_product(product):
        """Does the model have this product.
        """

    def get_product(product):
        """Return the given product.
        """


class IProductDocumentationProvider(Interface):
    """Provide documentation about products.
    """

    def has_product(product):
        """Does the model have this product.
        """

    def get_product(product):
        """Return the given product.
        """

# Test and testrunner

class ITestOptions(Interface):
    """Represent options to a test.
    """

    extension = Attribute(u"Name of extension")
    fix = Attribute(u"Schema fix values")
    vary = Attribute(u'Schema vary values')

class ISimpleTestExecutor(Interface):
    """A test executor wraps the execution of the test with needed
    actions (start/stop interpreter, set schema data...).
    """

    def run(result):
        """Run the test, using result as result storage.
        """

class IPolicyTestExecutor(ISimpleTestExecutor):
    """Policy test runner implementation.
    """

class IParameterTestExecutor(ISimpleTestExecutor):
    """Parameter test runner implementation.
    """

class ITestExecutor(ISimpleTestExecutor):
    """Regular test runner implementation.
    """


class ISimpleTest(Interface):
    """A test interface.
    """

    fix = Attribute(u"Schema of fixed used data")
    vary = Attribute(u"Schema of used data which can be changed")
    schema = Attribute(u"Whole schema of the test")

    def get_name():
        """Return the name of the test in lower case.
        """

    def run(result):
        """Run the test, putting result in result.
        """

    def expect_ok(*steps):
        """Run given steps expecting them to be successful.
        """

    def expect_ko(*steps):
        """Run given steps expecting the last one to fail.
        """

    def list_extensions():
        """Return available extension for the test.
        """

class ITest(ISimpleTest):
    """Test to be run using a RegularTestRunner.
    """

class IPolicyTest(ISimpleTest):
    """Policy test to be run using a PolicyTestRunner.
    """

class IParameterTest(ISimpleTest):
    """Parameter test to be run using a ParameterTestRunner.
    """

class ITestUploadFiles(Interface):
    """This marker interface says that's the test is going to upload
    files/images.
    """

class IDataProvider(Interface):
    """A DataProvider
    """

class IPolicyDataProvider(Interface):
    """A PolicyDataProvider
    """

class IScenario(Interface):
    """A Scenario
    """

class IScript(Interface):
    """A Script
    """

class ILogical(Interface):
    """A Logical Model
    """

class IPhysical(Interface):
    """A Physical Model
    """

class IExpectedStateGenerator(Interface):
    """An ExpectedStateGenerator
    """
