# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import datetime
import logging
import sys
import os
import os.path
import time, codecs
import atexit
from shutil import rmtree
from xml.sax.saxutils import quoteattr

from zope.interface import implements
from zope.event import notify
from zope.component import provideHandler
from zope.component import adapter

from funittest.utils import FormatExceptionInfo
from funittest.utils import normalizeClassName
from funittest.core.configuration import retrieveConfiguration
from funittest.core.interfaces import ILogger
from funittest.core.interfaces import ITestLogger
from funittest.core.interfaces import ISimpleTest
from funittest.core.events import ITestStarting
from funittest.core.events import ITestEnding

def _currentTimeRFC822():
    return time.strftime("%a, %d %b %Y %T %z", time.gmtime())

# Log for one run are in this sub-directory.
SUBLOG_NAME = 'report'
LOCK_FILE = 'funittest.lock'

def _getLogPath(full=True, extras=None, name=None):
    """Return default log path, FUNITTEST_LOG, config or current
    working directory.

    If extra is given it's append to the path log as directory (and
    create if non-existant), and if name is given as well, it's append
    to the path log as filename.
    """

    # First try env variable
    base_path = os.environ.get('FUNITTEST_LOG', None)

    # After, try configuration
    if base_path is None:

        config = retrieveConfiguration()
        if config:
            log = config.log()
            if log and log.has_key('directory'):
                base_path = log['directory']

    # No log path, use current directory
    if base_path is None:
        base_path = os.getcwd()
    else:                       # Clean path for windows
        base_path = os.path.abspath(base_path)

    if not os.path.exists(base_path):
        os.makedirs(base_path)

    if full:
        # Give the full path to the log directory
        base_path = os.path.join(base_path, SUBLOG_NAME)

    if not (extras is None):
        # Extra directory specified: add and create them
        base_path = os.path.join(base_path, *extras)
        if not os.path.exists(base_path):
            os.makedirs(base_path)

    if not (name is None):
        # Filename given, add it to the path
        base_path = os.path.join(base_path, name)

    return base_path



class TestFilter(object):
    """Filter for TestFileHandler, to be sure to use the handler only
    if there is a file.
    """

    def __init__(self, handler):
        self.handler = handler

    def filter(self, record):
        return not (self.handler.baseFilename is None)


class TestFileHandler(logging.FileHandler):
    """Generic test-only logger.
    """

    implements(ITestLogger)

    def __init__(self, extension, encoding='UTF-8'):
        logging.StreamHandler.__init__(self)
        self.baseFilename = None
        self.extension = extension
        self.encoding = encoding
        self.addFilter(TestFilter(self))

    def start(self, path_items):
        """Start the logging handler
        """

        filename = '%s%s' % (path_items[-1], self.extension)
        self.baseFilename = _getLogPath(extras=path_items[:-1],
                                        name=filename)
            
        if self.encoding:
            self.stream = codecs.open(self.baseFilename, 'w', self.encoding)
        else:
            self.stream = open(self.baseFilename, 'w')

    def flush(self):
        """Conditionnaly flush the handler.
        """
        if self.baseFilename:
            logging.FileHandler.flush(self)

    def close(self):
        """Close the loging handler.
        """
        if self.baseFilename:
            self.flush()
            self.stream.close()
            self.baseFilename = None

    def path(self):
        """Return the path to the log.
        """
        return self.baseFilename


class OPMLTagOpened(object):
    """An OPML tag have been opened.
    """

class OPMLTagClosed(object):
    """An OPML tag has been closed.
    """

@adapter(OPMLTagOpened)
def open_opml(event):
    opml = logger.retrieveLogger('opml')
    if opml:
        opml.opened_outline += 1
    
provideHandler(open_opml)

@adapter(OPMLTagClosed)
def close_opml(event):
    opml = logger.retrieveLogger('opml')
    if opml:
        opml.opened_outline -= 1

provideHandler(close_opml)


class OPMLFileHandler(TestFileHandler):
    """Save logs as OPML logs.
    """

    opened_outline = 0

    def start(self, path_items):
        TestFileHandler.start(self, path_items)
        print >>self.stream, """<?xml version="1.0" encoding="UTF-8"?>
<opml version="2.0">
   <head>
      <title>Funittest OPML Export</title>
      <dateCreated>%s</dateCreated>
   </head>
   <body>""" % _currentTimeRFC822()

    def close(self):
        assert(self.opened_outline > -1)
        if self.baseFilename:
            while self.opened_outline:
                print >>self.stream, '</outline>'
                self.opened_outline -= 1
            print >>self.stream, '</body></opml>'
            TestFileHandler.close(self)


class OPMLFormatter(logging.Formatter):
    """Format log as OPML log.
    """

    def format(self, record):
        """When the message is empty, switch to the closing opml tag, and
        otherwise use the opening opml tag.
        """

        if not hasattr(record, 'message') or not record.message:
            self._fmt = '</outline>'
            notify(OPMLTagClosed())
        else:
            record.message = quoteattr(str(record.message)).replace('"', "'")
            record.msg = record.message
            record.args = {}
            self._fmt = '<outline text="%%(message)s" created="%s">' % _currentTimeRFC822()
            notify(OPMLTagOpened())
        return logging.Formatter.format(self, record)


class SeleniumIDEFileHandler(TestFileHandler):
    """Save logs as Selenuim IDE Test.
    """

    def start(self, path_items):
        TestFileHandler.start(self, path_items)
        print >>self.stream, """<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<title>%s</title>
</head>
<body>
<table cellpadding="1" cellspacing="1" border="1">
<thead>
<tr><td rowspan="1" colspan="3">%s</td></tr>
</thead><tbody>""" % (path_items[-1], path_items[-1])

    def close(self):
        if self.baseFilename:
            print >>self.stream, """</tbody></table>
</body>
</html>"""
            TestFileHandler.close(self)

class SelenuimIDEFormatter(logging.Formatter):
    """Format log as Selenuim IDE test.
    """

    def format(self, record):
        """Format record correctly
        """
        if (('method' in record.args) and
            ('args' in record.args) and
            ('interpreter' in record.args)):
            return """<tr>
    <td>%s</td>
    <td>%s</td>
    <td></td>
</tr>""" % (record.args['method'], record.args['args'])
        return ''


class AllFilter(object):
    """Filter out all messages. Nothing passes this Filter.
    """
    def __init__(self, handler):
        self.handler = handler

    def filter(self, record):
        return False

class InternalFilter(object):
    """Filter out all internal messages. Take care to let pass through
       meta messages. For example the statistics about the number of
       tests passed or failed needs to be shown after a test run.
    """
    def __init__(self, handler):
        self.handler = handler

    def filter(self, record):
        from funittest import interpreter
        if not interpreter.wrappedinterpreter:
            return
        if record.getMessage().strip() == "":
            return False
        if "is_meta" in record.args:
            return True
        return False

class NothingFilter(object):
    def __init__(self, handler):
        self.handler = handler

    def filter(self, record):
        if record.getMessage().strip() == "":
            return False
        else:
            return True

class Logger(object):
    """Logger object.

    This object initialize logger settings, and provides convenient
    method to do logging.
    """

    implements(ILogger)

    def __init__(self, log_to, log_name=None,
                 level=logging.DEBUG, name='Funittest'):
        backupCount = 10

        # Overrides values with configuration file
        config = retrieveConfiguration()
        log_config = config.log()
        if 'backup_count' in log_config:
            backupCount = int(log_config['backup_count'])
        if 'to' in log_config:
            log_to = log_config['to']
        if 'name' in log_config:
            log_name = log_config['name']
        if 'logger' in log_config:
            name = log_config['logger']
        if 'level' in log_config:
            valid = ['CRITICAL', 'DEBUG', 'ERROR', 'FATAL', 'INFO', 'WARNING']
            value = log_config['level'].upper()
            if not (value in valid):
                raise ValueError, 'Invalid log level %s' % value
            level = getattr(logging, value)
    
        self._lock_file = None
        self._handlers = dict()
        self._logger = logging.getLogger(name)
        if self._logger.handlers:
            # Already setup, we can't make a new one, since it won't
            # have any handlers available.
            raise RuntimeError, 'Logger "%s" already initialized' % name

        self._lock_file = _getLogPath(name=LOCK_FILE)
        if os.path.isfile(self._lock_file):
            # Log directory already used.
            raise RuntimeError, 'Log directory already used. Delete the lock file "%s" by hand.' % self._lock_file

        # Move log files up one level, starting from the end
        base_path = _getLogPath(full=False)
        for i in range(backupCount - 1, 0, -1):
            sfn = os.path.join(base_path, "%s.%d" % (SUBLOG_NAME, i))
            dfn = os.path.join(base_path, "%s.%d" % (SUBLOG_NAME, i + 1))
            if os.path.exists(sfn):
                if os.path.exists(dfn):
                    rmtree(dfn)
                os.rename(sfn, dfn)
        log_path = _getLogPath()
        if os.path.exists(log_path):
            # Now that there is one spot free, move the report up
            os.rename(log_path, os.path.join(base_path, "%s.1" % SUBLOG_NAME))
        # Create a new report folder
        os.mkdir(log_path)
        # Create the lock file
        lock = open(self._lock_file, "w")
        lock.write(name)
        lock.close()
        # Base log name
        log_path = os.path.join(log_path, log_name)

        if log_to.count("console"):
            hdlr = logging.StreamHandler(strm=sys.stdout)
            self._handlers['console'] = hdlr
            self._logger.addHandler(hdlr)

        if log_to.count("file"):
            hdlr = logging.FileHandler('%s.log' % log_path)
            formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
            hdlr.setFormatter(formatter)
            self._handlers['file'] = hdlr
            self._logger.addHandler(hdlr)

        if log_to.count("opml"):
            hdlr = OPMLFileHandler('.opml')
            formatter = OPMLFormatter()
            hdlr.setFormatter(formatter)
            self._handlers['opml']  = hdlr

        if log_to.count("side"):
            hdlr = SeleniumIDEFileHandler('_side.html')
            formatter = SelenuimIDEFormatter()
            hdlr.setFormatter(formatter)
            self._handlers['side']  = hdlr
        
        self._logger.setLevel(level)

    def startTestLogging(self, path_items):
        """Start to log a new test.
        """
        for _, hdlr in self._handlers.items():
            if ITestLogger.providedBy(hdlr):
                hdlr.start(path_items)
                self._logger.addHandler(hdlr)

    def closeTestLogging(self):
        """End of test, close logger.
        """
        for _, hdlr in self._handlers.items():
            if ITestLogger.providedBy(hdlr):
                self._logger.removeHandler(hdlr)
                hdlr.close()

    def retrieveLogger(self, name):
        """Access to the given logger.
        """
        return self._handlers.get(name, None)
    
    def close(self):
        """Close the logger.
        """
        for hdlr in self._handlers.values():
            self._logger.removeHandler(hdlr)
            hdlr.close()
        if self._lock_file:
            if os.path.isfile(self._lock_file):
                os.unlink(self._lock_file)


    # OLD

    def start_preset(self, name):
        kwargs = {}
        kwargs['name']=name
        message = 'Run preset "%(name)s"'
        self._logger.info(message, kwargs)

    def end_preset(self):
        self._logger.info("")

    def start_tests(self, tests):
        kwargs = {}
        kwargs['time']= datetime.datetime.now().isoformat()
        kwargs['number']=len(tests)
        message = 'Run "%(number)s" tests, starting "%(time)s"'
        self._logger.info(message, kwargs)

    def end_tests(self, ):
        self._logger.info("")

    def start_test(self, test):
        kwargs = {}
        kwargs['name'] = test.get_name()
        if hasattr(test, "extension"):
            kwargs['extension'] = test.extension.split('test_')[-1]
        else:
            kwargs['extension'] = "main"
        message = 'Test "%(name)s" extension "%(extension)s"'
        self._logger.info(test)
        self._logger.info('')
        self._logger.info(message, kwargs)


    def end_test(self):
        self._logger.info("")

    def start_setup(self):
        message = 'SetUp'
        self._logger.info(message)

    def end_setup(self):
        self._logger.info("")

    def start_test_step(self, step):
        kwargs = {}
        kwargs['step'] = step
        message = "Test step %(step)s"
        self._logger.info(message, kwargs)

    def end_test_step(self):
        self._logger.info("")

    def test_no_error(self):
        message = "Expected error didn't occur."
        self._logger.info(message)
        self._logger.info("")

    def test_error(self):
        kwargs = {}
        kwargs['traceback'] = FormatExceptionInfo()
        message = "Error: %(traceback)s"
        self._logger.error(message, kwargs)
        self._logger.error("")

    def start_scenario(self, instance):
        kwargs = {}
        kwargs['model'] = normalizeClassName(instance)
        message = 'scenarios.%(model)s()'
        self._logger.info(message, kwargs)

    def run_scenario(self, scenario):
        kwargs = {}
        kwargs['model'] = normalizeClassName(scenario)
        message = 'Run scenario: %(model)s'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def run_scenario_extension(self, extension, configuration):
        kwargs = {}
        kwargs['extension'] = extension
        kwargs['configuration'] = configuration
        message = 'Run scenario extension: %(extension)s %(configuration)s'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def dataprovider_key_not_found(self, dataprovider, key):
        kwargs = {}
        kwargs['dataprovider'] = dataprovider
        kwargs['key'] = key
        message = 'Dataprovider "%(dataprovider)s" key "%(key)s" not found'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def end_scenario(self):
        self._logger.info("")

    def start_logical_method(self, instance, method_name, args, kwargs):
        kwargs = {}
        kwargs['model'] = normalizeClassName(instance)
        kwargs['name'] = method_name
        kwargs['args'] = str(args)
        kwargs['kwargs'] = str(kwargs)
        message = 'logical.%(model)s.%(name)s(%(args)s,%(kwargs)s'
        self._logger.info(message, kwargs)

    def end_logical_method(self, instance, method_name, args, kwargs):
        self._logger.info("")

    def log_annotation(self, annotation):
        kwargs = {}
        kwargs['annotation']=annotation 
        message = 'annotation: %s'
        self._logger.info(message, kwargs)
        self._logger.info("")
    
    def start_interpreter_method(self, method, args, kw):
        kwargs={}
        kwargs['interpreter']=True
        kwargs['method']=method
        kwargs['args']=''
        for arg in args:
            try:
                arg = str(arg)
            except:
                arg.encode('ascii', 'ignore')
            if type(arg) == type(""):
                kwargs['args']=kwargs['args'] + '"' + arg + '",'
            else:
                kwargs['args']=kwargs['args'] + arg + ","
        if kwargs['args'].endswith(","):
            kwargs['args'] = kwargs['args'][:-1]
        if kw:
            kwargs['kw']=str(kw)
            message = "interpreter.%(method)s(%(args)s, %(kw)s)"
        else:
            message = "interpreter.%(method)s(%(args)s)"
    
        self._logger.info(message, kwargs)

    def end_interpreter_method(self, method, args, kw):
        self._logger.info("")

    def start_behaviour_method(self, method, args, kw):
        kwargs={}
        kwargs['method']=method
        kwargs['args']=args
        kwargs['kw']=kw
        message = "behaviour.%(method)s(%(args)s, %(kw)s)"
        self._logger.info(message, kwargs)

    def end_behaviour_method(self, method, args, kw):
        self._logger.info("")

    def start_generate_expected_state(self, method, args):
        kwargs={}
        try:
            kwargs['method'] = method.im_func.func_name
        except:
            kwargs['method'] = method.func_name
        try:
            kwargs['class'] = method.im_class.__module__
        except:
            kwargs['class'] = method.__module__        
        kwargs['args'] = args
        message = "expect.%(class)s.%(method)s(%(args)s)"
        self._logger.info(message, kwargs)

    def end_generate_expected_state(self, on_method, args):
        self._logger.info("")

    def start_expected_state(self, state):
        kwargs={}
        kwargs['state'] = str(state.items())
        message = "expected state: %(state)s"
        self._logger.info(message, kwargs)

    def end_expected_state(self, state):
        self._logger.info("")

    def model_registration(self, model, name, product, origin):
        kwargs={}
        kwargs['model']=model
        kwargs['name']=name
        kwargs['product']=product
        kwargs['origin']=origin
        message = 'register model "%(model)s" with name "%(name)s" for product "%(product)s" from "%(origin)s"'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def expected_state_error(self, comparison, expectedstategenerators, method_name):
        kwargs={}
        kwargs['kwargs']=comparison
        kwargs['expectedstategenerators']=str(expectedstategenerators)
        kwargs['method_name']=method_name
        message = 'expected state error: %(kwargs)s, expected state generators: %(expectedstategenerators)s, method_name: %(method_name)s'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def start_verification(self, method, args):
        kwargs={}
        kwargs['method']=method.im_func.func_name
        kwargs['class']=method.im_class.__module__
        kwargs['args']=args
        message = "%(class)s.%(method)s(%(args)s)"
        self._logger.info(message, kwargs)

    def end_verification(self, method, args):
        self._logger.info("")

    def load_overarching_models(self):
        message = 'load overarching models'
        self._logger.info(message)
        self._logger.info("")
        
    def load_models_for_version(self, version):
        kwargs = {}
        kwargs['version'] = version
        message = 'load models for version %(version)s'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def annotation(self, annotation):
        kwargs = {}
        kwargs['annotation'] = annotation
        message = 'annotation: %(annotation)s'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def test_time(self, startTime, stopTime):
        kwargs = {}
        dateFormat = "%a, %d %b %Y %H:%M:%S"
        startTime = datetime.datetime.fromtimestamp(startTime)
        stopTime = datetime.datetime.fromtimestamp(stopTime)
        kwargs['start'] = startTime.strftime(dateFormat)
        kwargs['stop'] = stopTime.strftime(dateFormat)
        takenDelta = stopTime - startTime
        taken = datetime.time(takenDelta.seconds / 3600,
                              (takenDelta.seconds / 60) % 60,
                              takenDelta.seconds % 60)
        kwargs['total'] = taken.strftime("%Hh %Mm %Ss")
        kwargs['is_meta']=True
        message = "\nTotal time: %(total)s. Start at: %(start)s Stop at: %(stop)s"
        self._logger.info(message, kwargs)
        self._logger.info("")

    def test_result(self, result):
        kwargs = {}
        kwargs['failures'] = len(result.failures)
        kwargs['errors'] = len(result.errors)
        kwargs['success'] = int(result.testsRun) - len(result.failures) - len(result.errors)
        kwargs['is_meta']=True
        message = "Result: Failures: %(failures)d, Errors: %(errors)d, Success: %(success)d"
        self._logger.info(message, kwargs)
        self._logger.info("")

    def selenium_start_offline(self):
        kwargs = {}
        kwargs['is_meta'] = True
        message = "Start in offline mode (No connection to Selenium Remote Control)"
        self._logger.info(message, kwargs)
        self._logger.info("")

    def selenium_start_online(self, browserStartCommand, browserURL):
        kwargs = {}
        kwargs['browserStartCommand'] = browserStartCommand
        kwargs['browserURL'] = browserURL
        message = "Start in online mode. %(browserStartCommand)s %(browserURL)s)"
        self._logger.info(message, kwargs)
        self._logger.info("")

    def selenium_session_established(self, sessionId):
        kwargs = {}
        kwargs['sessionId'] = sessionId
        message = "Session with Selenium Remote Control established with sessionId %(sessionId)s"
        self._logger.info(message, kwargs)
        self._logger.info("")

    def selenium_error(self, data, reason, commandString, traceback):
        kwargs = {}
        kwargs['data'] = str(data)
        kwargs['reason'] = str(reason)
        kwargs['commandString'] = str(commandString)
        kwargs['traceback'] = traceback
        message = "Selenium Error: %(data)s, %(reason)s\n%(commandString)s\n%(traceback)s"
        self._logger.error(message, kwargs)
        self._logger.error("")

    def exception(self):
        kwargs = {}
        kwargs['traceback'] = FormatExceptionInfo()
        message = "Error: %(traceback)s"
        self._logger.error(message, kwargs)
        self._logger.error("")

    def login_as_administrator(self, admin):
        kwargs = {}
        kwargs['admin'] = admin
        message = "Log in as administrator %(admin)s"
        self._logger.error(message, kwargs)
        self._logger.error("")

    def login_as_administrator_failed(self, admin):
        kwargs = {}
        kwargs['admin'] = admin
        message = "Failed to log in as administrator %(admin)s. Go to the ZMI and change the password in the acl_users object at root level."
        self._logger.error(message, kwargs)
        self._logger.error("")

    def start_establish_dataprovider(self, dataprovider):
        kwargs = {}
        kwargs['dataprovider'] = normalizeClassName(dataprovider)
        kwargs['is_meta'] = True
        message = 'Start establishing the dataprovider "%(dataprovider)s"'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def end_establish_dataprovider(self, dataprovider):
        kwargs = {}
        kwargs['dataprovider'] = normalizeClassName(dataprovider)
        message = 'Dataprovider "%(dataprovider)s" established'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def run_policytest(self, policytest, filename, preset):
        kwargs = {}
        kwargs['policytest_id'] = policytest.get_name()
        kwargs['filename'] = filename
        kwargs['preset'] = preset
        message = 'Running Policy Test "%(policytest_id)s", "%(filename)s", "(preset)s"'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def run_parametertest(self, parametertest, test_id, test_parameters):
        kwargs = {}
        kwargs['parametertest_id'] = parametertest.get_name()
        kwargs['test_id'] = test_id
        kwargs['test_parameters'] = test_parameters
        message = 'Running Parameter Test "%(parametertest_id)s", "%(test_id)s", "%(test_parameters)s"'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def log_printErrors(self, result):
        class Writeln:
            text = ""
            def write(self, arg):
                self.text = self.text + arg
            def writeln(self, arg=None):
                if arg:
                    self.write(arg)
                self.write('\n')
        writeln = Writeln()
        result.stream = writeln
        result.printErrors()
        kwargs = {}
        kwargs['errors'] = writeln.text
        kwargs['is_meta'] = True
        message = '%(errors)s'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def log_test_comment(self, comment):
        kwargs = {}
        kwargs['comment'] = comment
        message = '%(comment)s'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def log_established_dataproviders(self, established):
        kwargs = {}
        kwargs['established'] = established
        kwargs['is_meta'] = True
        message = '%(established)s DataProviders established'
        self._logger.info(message, kwargs)
        self._logger.info("")

    def log_usage(self, usage):
        kwargs = {}
        kwargs['usage'] = usage
        kwargs['is_meta'] = True
        message = "%(usage)s"
        self._logger.info(message, kwargs)
        self._logger.info("")

    def log_optionparser_error(self, progname, error):
        kwargs = {}
        kwargs['progname'] = progname
        kwargs['error'] = error
        kwargs['is_meta'] = True
        message = "%(progname)s: error: %(error)s\n"
        self._logger.error(message, kwargs)
        self._logger.error("")

    def log_seleniumrc_not_found(self, seleniumpath):
        kwargs = {}
        kwargs['seleniumpath'] = seleniumpath
        kwargs['is_meta'] = True
        message = """The Selenium RC launch script, 'seleniumrc', can't be found at %(seleniumpath)s. Be sure to select a buildout profile, and run it before using this script."""
        self._logger.error(message, kwargs)
        self._logger.error("")

logger = Logger(log_to="console opml file",
                log_name="funittest",
                name="FunittestResult")
atexit.register(logger.close)

def start_test(test, event):
    logger.startTestLogging(event.test_ids)

provideHandler(start_test, [ISimpleTest, ITestStarting,])

def end_test(test, event):
    logger.closeTestLogging()

provideHandler(end_test, [ISimpleTest, ITestEnding,])

    
