# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

## Contains all exceptions.

from utils import normalizeClassName
from funittest.core.interfaces import IPolicyTest

class ModelOfProductImportError(Exception):
    """Exception raised when a product cannot be loaded.
    """
    
    def __init__(self, product, model):
        self.product = product
        self.model = model
        
    def __str__(self):
        msg = 'No "%s" model for product "%s" found.'
        return msg % (self.model, self.product)

class InstanceExpected(Exception):
    """Exception raised when you register a model with something else
    than an instance.
    """

    def __init__(self, model, name):
        Exception.__init__(self)
        self.model = model
        self.name = name

    def __str__(self):
        msg = "Expected an instance when registering model, %s with name %s"
        return  msg % (self.model, self.name)


class EnforceDictionaryArguments(Exception):
    """Exception raised when you try to give data which is not
    provided by data provider to your scenario/logical step.
    """

    def __init__(self, instance, method_name, args, kwargs):
        Exception.__init__(self)
        self.instance = instance
        self.method_name = method_name
        self.args = args
        self.kwargs = kwargs

    def __str__(self):
        return ("Arguments are enforced to be dictionaries. " +
                "Use keyword parameters if necessary. " +
                "Logical Model: %s, method: %s, args: %s, kwargs: %s" % (normalizeClassName(self.instance), 
                                                                         self.method_name, 
                                                                         str(self.args), 
                                                                         str(self.kwargs)))

class OnMethodExpectedStateError(Exception):
    """Exception raised when an exception has occured in the on_method hook of an
       expected state generator.
    """

    def __init__(self):
        Exception.__init__(self)

    def __str__(self):
        return "Error detected in the on_ method of an Expected State Generator"

class ExpectedStateVerificationError(Exception):
    """Exception raised when a state is not validated.
    """

    def __init__(self, info):
        Exception.__init__(self)
        self.info = info

    def __str__(self):
        return "Expected State Verification Error: %s" % self.info


class EnforceModelClassNaming(Exception):
    """Expression raised then a filename doesn't match a class name
    inside.
    """

    def __init__(self, class_name, module_name):
        Exception.__init__(self)
        self.class_name = class_name
        self.module_name = module_name

    def __str__(self):
        msg = 'The name of the class "%s" in lower case should be the name of its module "%s"'
        return  msg % (self.class_name, self.module_name)


class EnforceKeyInDefaults(Exception):
    """Exception raised when a provided data doesn't have any default
    values.
    """

    def __init__(self, dataprovider, defaults, key):
        Exception.__init__(self)
        self.defaults = defaults
        self.key = key

    def __str__(self):
        msg = 'Expected the key "%s" to be in the defaults: %s'
        return  msg % (self.key, str(self.defaults))


class EnforceKeyProvided(Exception):

    def __init__(self, model, key):
        Exception.__init__(self)
        self.model = model
        self.key = key

    def __str__(self):
        msg = 'Expected the key "%s" to be provided by the %s model "%s"'
        model_type = "(unknown model type)"
        if IPolicyTest.providedBy(self.model):
            model_type = "policy test"
        else:
            model_type = "dataprovider"
        return  msg % (self.key, model_type, normalizeClassName(self.model))

class AlreadyDataProvidedUnderThisKey(Exception):

    def __init__(self, dataprovider, key):
        Exception.__init__(self)
        self.dataprovider = dataprovider
        self.key = key

    def __str__(self):
        msg = 'There already is data provided under the key "%s" in the dataprovider "%s"'
        return  msg % (self.key, normalizeClassName(self.dataprovider))

class NothingToChooseFrom(Exception):

    def __init__(self, dataprovider):
        Exception.__init__(self)
        self.dataprovider = dataprovider

    def __str__(self):
        msg = 'Nothing to choose from in this dataprovider "%s"'
        return  msg % normalizeClassName(self.dataprovider)

class KeyMismatch(Exception):

    def __init__(self, dataprovider, key, value):
        Exception.__init__(self)
        self.dataprovider = dataprovider
        self.key = key
        self.value = value

    def __str__(self):
        msg = 'Key "%s" does not match to key "%s" in value for dataprovider %s'
        return  msg % (self.key, self.value[self.dataprovider.key], normalizeClassName(self.dataprovider))

class AtLeastOneStepRequired(Exception):
    """The test/scenario doesn't contain any steps.
    """

    def __str__(self):
        return "At lease one step is required."

class NoTestExecutorForTest(Exception):
    """There is no found test executor to run the test.
    """

    def __init__(self, test):
        self.test = test

    def __str__(self):
        return "No test executor found to execute this test %s" % self.test.get_name()

class ScenarioExtensionNotFound(Exception):
    """The extension method 
    """
    
    def __init__(self, scenario, extension):
        self.scenario = scenario
        self.extension = extension
        extensions = scenario.list_extensions()
        if extensions:
            self.available = extensions
        else:
            self.available = []
    
    def __str__(self):
        return "The scenario does not have an extension called %s. Available extensions: " % self.extension + ', '.join(self.available)


class FailedScenarioConfiguration(Exception):
    """The scenario couldn't be configured
    """
    
    def __init__(self, scenario, schema, configuration_key):
        self.scenario = scenario
        self.schema = schema
        self.configuration_key = configuration_key
    
    def __str__(self):
        return '''The scenario "%s" couldn't be configured for the schema "%s" with the key "%s"''' % (self.scenario.get_name(), normalizeClassName(self.schema), self.configuration_key)

class IncompatibleSchema(Exception):
    def __init__(self, schema, key):
        self.schema = schema
        self.key = key

    def __str__(self):
        return "Key \"%s\" not found in Schema. Schema keys:" % self.key + ', '.join(self.schema.keys())

class ConflictingSchemaConfiguration(Exception):
    def __init__(self, key):
        self.key = key

    def __str__(self):
        return "Can't both fix and vary Schema key %s" % self.key

class SeleniumResponseError(Exception):
    def __init__(self, data, reason, commandString):
        self.data = str(data)
        self.reason = str(reason)
        self.commandString = str(commandString)

    def __str__(self):
        return "Selenium Response Error: %s, %s, %s" % (self.data, self.reason, self.commandString)

class ErrorExpectedInLastStep(Exception):

    def __init__(self, test, step):
        self.test = test
        self.step = step

    def __str__(self):
        return "Error expected in step %s of test %s" % (self.step, self.test.get_name())

class WrongSchema(Exception):
    def __init__(self, schema, value):
        self.schema = schema
        self.value = value

    def __str__(self):
        error = """The schema for the dataprovider '%s' enforces the use of the default keys %s, but the provided value, '%s', does not comply with this rule. Make sure you use the right dataprovider.""" % (normalizeClassName(self.schema), str(self.schema.defaults.keys()), str(self.value))
        return error

class ValueNotDict(Exception):
    def __init__(self, key, value):
        self.key = key
        self.value = value

    def __str__(self):
        error = """The value for the key %s is not a dictionary: %s.""" % (str(self.key), str(self.value))
        return error

class NotPartOfSchema(Exception):
    def __init__(self, scenario, schema, key):
        self.schema = schema
        self.key = key
        self.scenario = scenario

    def __str__(self):
        error = """The key "%s" is not part of the schema "%s" of the scenario "%s".""" % (str(self.key), str(self.schema), self.scenario)
        return error

class EnforceKeywordArguments(Exception):
    def __init__(self, args):
        self.args = args

    def __str__(self):
        error = """Pass the following args as keyword arguments so that they can be associated to the schema: %s.""" % (str(self.args))
        return error

class ConfigurationError(Exception):
    def __init__(self, key):
        self.key = key

    def __str__(self):
        return "Configuration error: dataprovider %s is empty" % self.key

class MissingStep(Exception):
    def __init__(self, scenario, step):
        self.scenario = scenario
        self.step = step

    def __str__(self):
        error = '''The step "%s" is missing in the scenario "%s"''' % (self.step, self.scenario.get_name())
        return error

class PresetValueListExpected(Exception):

    def __init__(self, policytest, arguments):
        Exception.__init__(self)
        self.policytest = policytest
        self.arguments = arguments

    def __str__(self):
        msg = 'Expected a list in all values "%s" of the preset  "%s" in "%s"'
        return  msg % (self.arguments['preset'], self.arguments['id'], normalizeClassName(self.policytest))

class OptionParserError(Exception):
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return '''%s''' % self.msg
