# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import traceback
from docutils.core import publish_parts

from funittest.log import logger
from funittest.problems import ModelOfProductImportError
from funittest.problems import InstanceExpected
from funittest.problems import EnforceModelClassNaming
from funittest.utils import normalizeClassName


def verifymodelclassname(model):
    """Check that class Content is in a lfm_content module for instance.
    """
    # For a model.__module__ that equals 
    #     funittest.lib.plone214.plone.state.esg_content
    # The module_name and the class_name is
    #     content
    module_name = model.__module__.split(".")[-1].split("_")[-1]
    class_name = normalizeClassName(model)
    if module_name != class_name:
        raise EnforceModelClassNaming(class_name, module_name)
    return True

class Product(object):
    """A product contains model, which is class instances.
    """

    def __init__(self, product_name):
        self.models = []
        self.product_name = product_name

    def get_name(self):
        return self.product_name

    def get_models(self):
        models = []
        for m in self.models:
            if hasattr(m, 'precondition'):
                # Models that that don't fulfill the precondition can't be run
                # For example, Plone 2.5 does not yet have the Security Control
                # Panel, so the scenario of editing it is excluded because
                # the precondition method returns false.
                if m.precondition():
                    models.append(m)
            else:
                models.append(m)
        return models

    def get_models_info(self, html=True):
        """Return models information for this registery (documentation).
        """

        def get_info(model):
            doc = model.__doc__
            if doc and html:
                doc = publish_parts(doc, writer_name='html')['fragment']
            return {'name': normalizeClassName(model),
                    'doc': doc}

        return [get_info(model) for model in self.get_models()]

    def register_model(self, model, name, origin, module):
        if not hasattr(model, '__class__'):
            raise InstanceExpected(model, name)
        model.__parent__ = self # To be able to know from which
                                # product a model comes from
        model.__origin = origin
        model.__name = name
        model.__module = module
        for m in self.models:
            if m.__name == model.__name:
                self.models.remove(m)
        self.models.append(model)

    def reset_models(self):
        self.models=[]

    def __getattr__(self, model):
        candidates = []
        for m in self.models:
            if m.__name == model.lower():
                if hasattr(m, 'precondition'):
                    # Models that that don't fulfill the
                    # precondition can't be run For example,
                    # Plone 2.5 does not yet have the Security
                    # Control Panel, so the scenario of
                    # editing it is excluded because the
                    # precondition method returns false.
                    if m.precondition():
                        return m
                else:
                    return m
        msg = 'Model name "%s" not found in models. Available models: %s'
        raise AttributeError, msg % (str(model), [m["name"] for m in self.get_models_info()])
 
    def __getitem__(self, model):
        return self.__getattr__(model)


class ModelRegistry(object):
    """The model registry contains product. It makes possible to
    access models directly or indirectly by product name.
    
    Generic models, like "users" and "navigation" can be accessed from the 
    model as an attribute:
    
         logical.plone.navigation.top()
    
    When registering global models, just pass in a product name with the value
    of None
    
    If you want to replace the standard Plone navigation, just register a new 
    navigation model after the Plone navigation model has been registered.
    
    Usually you want to have your own namespace for your product. In this case
    you pass in the product name, and your model will be made available as an 
    attribute like this:
    
         modeltype.myproductname.mymodel.myverb()
    """

    def __init__(self, name):
        self.by_product={}
        self.name = name

    def __repr__(self):
        return "Funittest model registry for %s models" % self.name

    def __str__(self):
        return self.name
    
    def reinit(self):
        self.by_product = {}

    def get_models(self):
        """Return models available for this registery.
        """
        models =[]
        for product in self.by_product.values():
            models.extend(product.get_models())
        return models

    def has_product(self, product_name):
        """Return true when the registery have data about the given product.
        """
        return self.by_product.has_key(product_name.lower())

    def get_product(self, product_name):
        """Lookup a product through the registery.
        """
        return self.by_product[product_name.lower()]

    def get_products_info(self):
        """Return information about available products.
        """

        def get_info(product):
            return {'name': product.get_name(),}

        return [get_info(product) for product in self.by_product.values()]

    def reset_product_models(self, product_name):
        if self.has_product(product_name):
            product = self.get_product(product_name)
            product.reset_models()

    def register_model(self, product_name, model, name, origin, module):
        """Register a new model in the registery.
        """
        if not hasattr(model, '__class__'):
            raise InstanceExpected(model, name)
        if not hasattr(model, "_TestCase__testMethodName"):
            model._testMethodName=None
            model._TestCase__testMethodName=None
        verifymodelclassname(model)
        logger.model_registration(self.name, name, product_name, origin)
        # Make the model available indirectly through the product name as
        # an attribute
        if self.has_product(product_name):
            # Get the existing product containing models
            product = self.by_product[product_name.lower()]
        else:
            # Create the product that will contain the models
            product = Product(product_name)
            product.__parent__ = self
            self.by_product[product_name.lower()] = product
        # Let the product handle making the model available as an attribute
        product.register_model(model, name, origin, module)

    def __getattr__(self, product):
        """Return the product for models.
        
        This method is called when get the product attribute from a model
        
        logical.myproduct
        
        and loads the newest 
        """
        if self.has_product(product):
            return self.get_product(product)

        from funittest.core.grokker import load_funittest_model
        if product == "__grok_module__":
            return None
        # Try loading the product from this Plone version
        modelpath = "funittest.lib.%s.%s" % (product.lower(), "newest")
        load_funittest_model(modelpath)
        if self.has_product(product):
            return self.get_product(product)
        # We didn't find any product by that name, so we raise an error.
        raise ModelOfProductImportError(product=product, model=self.name)

    def __getitem__(self, product):
        return self.__getattr__(product)

physical = ModelRegistry("physical")
dataprovider = ModelRegistry("dataprovider")
logical = ModelRegistry("logical")
scenarios = ModelRegistry("scenarios")
tests = ModelRegistry("tests")
scripts = ModelRegistry("scripts")
