# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import random
import time

from zope.interface import implements
from funittest.core.interfaces import IDataProvider
from funittest.core.interfaces import IPolicyDataProvider

from funittest.problems import EnforceKeyInDefaults
from funittest.problems import EnforceKeyProvided
from funittest.problems import AlreadyDataProvidedUnderThisKey
from funittest.problems import KeyMismatch
from funittest.problems import NothingToChooseFrom
from funittest.utils import normalizeClassName

class DataProvider(object):
    """Storage for data.
    """
    implements(IDataProvider)

    counter = 0 # Used for avoiding conflicts when randomizing ids

    def __init__(self, product, defaults, key):
        self.key=key # data is looked up with this key
        self.defaults = defaults
        assert(type(product)==type(""))
        assert(len(product)>0)
        self.product = product
        self.data={}
        # Currently expected to be established data. Before running a test,
        # it is assumed that the current data has been established.
        self.current = []
        # Should the data in the current variable be established when
        # running bin/preset.py?
        self.establish_current = False
        if not defaults.has_key(self.key) and not self.key is None:
            raise EnforceKeyInDefaults(self, self.defaults, self.key)

    def get_name(self):
        return str(self.product)

    def delete(self, key):
        del self.data[key]

    def __contains__(self, key):
        return key in self.keys()

    def keys(self):
        return self.data.keys()

    def provide(self, args):
        if not args.has_key(self.key):
            raise EnforceKeyProvided(self, self.key)
        if self.data.has_key(args[self.key]):
            raise AlreadyDataProvidedUnderThisKey(self, args[self.key])
        values = self.defaults.copy()
        values.update(args)
        key = values[self.key]
        self.data[key]=values

    def get(self, key):
        return self.data[key].copy()

    def has_key(self, key):
        return self.data.has_key(key)

    def __getitem__(self, key):
        return self.get(key)
    
    def set(self, key, value):
        if not value.has_key(self.key):
            raise EnforceKeyProvided(self, self.key)
        if not value[self.key] == key:
            raise KeyMismatch(self, key, value)
        self.data[key]=value

    def __setitem__(self, key, value):
        self.set(key, value)

    def update(self, key, value):
        if not value.has_key(self.key):
            raise EnforceKeyProvided(self, self.key)
        if not value[self.key] == key:
            raise KeyMismatch(self, key, value)
        self.data[key].update(value)

    def all(self):
        return self.data.values()

class PolicyDataProvider(DataProvider):
    """Data provider for policy test: provide test behaviour.
    """

    implements(IPolicyDataProvider)

    def __init__(self, product, test=None):
       """Create a data provider for policy test. 

       Product is the name of the product containing the data
       provider, test is the name of the test (in the same product)
       for which the data provider provides data. If test not
       provided, the same name that the data provider will be used for
       the test.
       """

       defaults = {'id': 'policytestid',
                   'file': '/the/path/to/the/file/containing/test/data.csv',
                   'preset': {"p1":[],
                              "p2":[]},
                   }
       DataProvider.__init__(self, product, defaults, 'id')
       if test is None:
           test = normalizeClassName(self)
       self.test = test

    def provide(self, args):
       """Provide provides data for the test as well.
       """
       DataProvider.provide(self, args)

       from funittest import tests

       test = tests[self.product][self.test]
       test.provide(args)

class Schema:
    """A schema is set as an attribute on a Scenario like this:
       
           schema = Schema({"user":dataprovider.plone.user})
       
       In this case, when you use the scenario, you should be able
       to access the _user attribute on the scenario instance, and the
       user should be coming from dataprovider.plone.user.
    """

    def __init__(self, dict=None):
        # We don't want to store a reference to any dataprovider instances,
        # as they can be swapped out dynamically
        #
        # So, we create a new dictionary composed of a tuple containing the
        # product name and the normalized class name.
        self.data = {}
        self.update(dict)

    def __getitem__(self, key):
        from funittest import dataprovider
        product, dp = self.data[key]
        return dataprovider[product][dp]

    def update(self, dict):
        if dict:
            for key, value in dict.items():
                product = value.get_name()
                self.data[key] = (product, normalizeClassName(value))

    def has_key(self, key):
        return self.data.has_key(key)
    
    def keys(self):
        return self.data.keys()

    def items(self):
        items = []
        for key in self.keys():
            items.append((key, self.__getitem__(key)))
        return items
    
