# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from funittest.registry import ModelRegistry
from funittest.utils import normalizeClassName
from funittest.log import logger
from funittest.problems import ExpectedStateVerificationError
from funittest import interpreter

class State(object):

    def __init__(self):
        self.state={}

    def compareTo(self, state):
        diffs = []
        for key in self.state.keys():
            # Start with the captured state and go through all keys
            if state.state.has_key(key):
                # Only compare if there is an expectation for that key
                if self.state[key] != state.state[key]:
                    # The expectation is not met, so add a  diff
                    diffs.append(str(key) + ": " + str(self.state[key]) + " != " + str(state.state[key]))
        if diffs:
            return (False,",".join(diffs))
        else:
            return (True,"")

    def set(self, key, value):
        self.state[key]=value
    
    def get(self, key):
        return self.state[key]

class VerificationManager(ModelRegistry):
    active = True
    expectedstate=None
    actualstate=None
    
    def expectedstategenerators(self, lfm):
        expectedstategenerators = []
        lfm_name = normalizeClassName(lfm)
        for esg in self.get_models():
            esg_name = normalizeClassName(esg)
            if lfm_name == esg_name:
                expectedstategenerators.append(esg)
        return expectedstategenerators

    def actionStarting(self, lfm, method_name, args):
        # The Verification Manager grabs the current state
        self.expectedstate = self.snapshotCurrentState(lfm, method_name)
        # Pass operation beginning message on to expected state generators
        self.notifyExpectedStateGenerators(lfm, method_name, args, self.expectedstate);
        
    def snapshotCurrentState(self, lfm, method_name):
        state=State()
        # We notify the expected state generators with the arguments and the expected state
        for esg in self.expectedstategenerators(lfm):
            esg.state = state
            esg.snapshot()
        return state

    def notifyExpectedStateGenerators(self, lfm, method_name, args, state):
        # We notify the expected state generators with the arguments and the expected state
        for esg in self.expectedstategenerators(lfm):
            # if the esg is interested in the action, it needs to implement
            # the method with the prefix 'on_'
            on_method = getattr( esg, 'on_' + method_name, None )
            if on_method:
                esg.state = self.expectedstate
                logger.start_generate_expected_state(on_method, args)
                result = apply(on_method, args) # kw omitted for now
                logger.end_generate_expected_state(on_method, args)
                logger.start_expected_state(esg.state.state)
                logger.end_expected_state(esg.state.state)

    def actionEnding(self, lfm, method_name, args):
        expectedstategenerators = self.expectedstategenerators(lfm)
        comparison_ok, comparison_info = self.expectedstate.compareTo(self.actualstate)
        if not comparison_ok:
            logger.expected_state_error(comparison_info, expectedstategenerators, method_name)
        for esg in expectedstategenerators:
            method = getattr( esg, method_name, None )
            if method:
                logger.start_verification(method, args)
                result = apply(method, args) # kw omitted for now
                logger.end_verification(method, args)
        return comparison_ok, comparison_info

    def off(self):
        self.active = False

    def on(self):
        self.active = True
        
    def is_verification_done(self):
        return self.active

    def verification_error(self, info):
        if not interpreter.is_dry_run():
            raise ExpectedStateVerificationError(info=info)

verification=VerificationManager("verification")

