# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os
import sys
import time

from exceptions import AssertionError

from unittest import _WritelnDecorator
from unittest import _TextTestResult

from zope.interface import implements
from zope.component import queryMultiAdapter

from funittest.log import logger
from funittest.problems import AtLeastOneStepRequired
from funittest.problems import NoTestExecutorForTest
from funittest.problems import ErrorExpectedInLastStep
from funittest.problems import ExpectedStateVerificationError
from funittest.problems import OnMethodExpectedStateError
from funittest.utils import normalizeClassName
from funittest.registry.dataprovider import Schema
from funittest.core.interfaces import ISimpleTestExecutor, ITestOptions
from funittest.core.interfaces import ITestUploadFiles
from funittest.core.configuration import get_browser_config

from funittest import interpreter

class TestOptions(object):
    """Represent context of a test.
    """

    implements(ITestOptions)

    def __init__(self, extension=None, vary=None, fix=None):
        if vary and fix:
            for key in vary:
                if key in fix:
                    msg = "Can't both fix and vary variable %s" % key
                    raise ValueError, msg

        self.extension = extension
        self.vary = vary
        self.fix = fix



class TestRunner(object):
    """TestRunner object, run all tests.
    """

    def __init__(self, tests, extension=None, vary=None, fix=None):
        """
        tests       - The tests to be run
        extension   - The extension to run, for example "3a"
                      "all" means running the the main test and all
                      extensions
        vary        - Choose configurations by using all variations of one
                      parameter
        fix         - Fix configuration parameter
        """
        for test in tests:
            if extension:
                if not extension in test.list_extensions() and extension != 'all':
                    msg = "Extension \"%s\" not found. Available extensions: %s"
                    raise KeyError, msg  % (str(extension), 
                                  ', '.join(test.list_extensions()))
            if fix:
                for f in fix:
                    if not f in test.schema.keys():
                        msg = "Variable \"%s\" not found. Available variables: %s"
                        raise  KeyError, msg % (f, ', '.join(test.fix.keys()))
            else:
                fix = []                # Be sure to be a list
            if vary:
                for key in vary:
                    if not key in test.schema.keys():
                        msg = "Variable \"%s\" not found. Available variables: %s"
                        raise  KeyError, msg % (key, ', '.join(test.vary.keys()))
            else:
                vary = []

        self.tests = tests
        self.options = TestOptions(extension, vary, fix)
        self.stream = _WritelnDecorator(sys.stderr)

    def __repr__(self):
        """Display testrunner information.
        """
        text = []
        text.append('=' * 72)
        text.append('Running %d test cases' % len(self.tests))
        text.append('=' * 72)
        text.append("\n")
        return '\n'.join(text)


    def run(self):
        """Run the tests, by adapting a test to a TextExecutor.
        """
        result = _TextTestResult(stream=self.stream, 
                                 descriptions=1, 
                                 verbosity=1)
        startTime = time.time()
        
        logger.start_tests(self.tests)
        for test in self.tests:
            executor = queryMultiAdapter((test, self.options,), 
                                         ISimpleTestExecutor)
            if executor is None:
                raise NoTestExecutorForTest(test)
            executor.run(result)
        logger.end_tests()
        logger.log_printErrors(result)
        stopTime = time.time()
        logger.test_time(startTime, stopTime)
        logger.test_result(result)

        # Return a correct status code.
        if result.errors:
            return 2
        if result.failures:
            return 1
        return 0

class TestInterpreterMixin(object):
    """Base class to run one test. This contains utility for managing
    the interpreter.
    """

    def start_test(self, test):
        """Method called before start any test: bring up the
        interpreter.
        """
        browser_type, browser_url = get_browser_config()
        if browser_type == 'firefox':
            if self._is_chrome_needed(test):
                browser_type = 'chrome'
        interpreter.setBrowserStartCommand("*" + browser_type)
        if browser_url:
            interpreter.setBrowserURL(browser_url)
        interpreter.start()
        interpreter.open('/')

    def end_test(self):
        """Method called after a test: shut down the interpreter.
        """
        interpreter.stop()

    def _is_chrome_needed(self, test):
        # Auto-detect when chrome mode is needed
        # Chrome mode is activated when there is a field that
        # ends in "_file"
        chrome_needed = 0
        if ITestUploadFiles.providedBy(test):
            chrome_needed = 1
        return chrome_needed

class BaseTest(object):
    """Base class for Test objects.
    """


    def fix(self):
        """Default empty schema for fix
        """
        return Schema()

    fix = property(fix)         # Python 2.3 compatible

    def vary(self):
        """Default empty schema for vary
        """
        return Schema()

    vary = property(vary)

    def schema(self):
        """Default schema: fix + vary
        """
        schema = Schema()
        schema.update(self.fix)
        schema.update(self.vary)
        return schema

    schema = property(schema)


    def get_name(self):
        """Return name of the test.
        """
        return normalizeClassName(self)

    def expect_ko(self, *steps):
        remainingsteps = len(steps)
        if remainingsteps == 0:
            raise AtLeastOneStepRequired()
        for step in steps:
            logger.start_test_step(step)
            if remainingsteps == 1:
                expected_error_ocurred = False
                # The interpreter should not pause on error, 
                # The error is expected, so pause does not make any sense
                remembered_pause_on_ko = interpreter.get_pause_on_ko()
                # Force no pause, we set back the initial value directly after
                interpreter.set_pause_on_ko(False)
                try:
                    getattr(self, 'step_' + str(step))()
                except ExpectedStateVerificationError:
                    # This error is raised in expected state generators
                    # when the comparison of the expected state fails
                    expected_error_ocurred = True
                except OnMethodExpectedStateError:
                    # This error is raised in the on_ method of expected
                    # state generators when a method like self.failIf is
                    # called
                    expected_error_ocurred = True
                except AssertionError:
                    # This error is raised in tests when methods
                    # like self.failIf are called
                    expected_error_ocurred = True
                except Exception, details:
                    # All other errors have to be programming errors that
                    # need to arrive on the surface.
                    logger.exception()
                # Set back the initial value
                interpreter.set_pause_on_ko(remembered_pause_on_ko)
                if expected_error_ocurred:
                    logger.end_test_step()
                else:
                    logger.test_no_error()
                    logger.end_test_step()
                    if interpreter.is_dry_run():
                        pass
                    else:
                        raise ErrorExpectedInLastStep(self, step)
            else:
                getattr(self, 'step_' + str(step))()
                logger.end_test_step()
            remainingsteps -= 1

    def expect_ok(self, *steps):
        for step in steps:
            logger.start_test_step(step)
            try:
                getattr(self, 'step_' + str(step))()
            except:
                logger.test_error()
                logger.end_test_step()
                raise
            logger.end_test_step()

    def list_extensions(self):
        """List availables extensions.
        """
        extensions = []
        for m in dir(self):
            if m.startswith('test_'):
                extensions.append(m[len('test_'):])
        extensions.sort()
        return extensions


