import os
import os.path
import sys
import time

from zope.interface import implements
from zope.component import adapts, provideAdapter
from funittest import objectEventNotify
from funittest import interpreter
from funittest import dataprovider
from funittest import scenarios

from funittest.problems import IncompatibleSchema
from funittest.runner.policytestresult import PolicyTestResult
from funittest.runner.testrunner import SimpleTestExecutor
from funittest.core.interfaces import IPolicyTest
from funittest.core.interfaces import IPolicyTestExecutor
from funittest.core.interfaces import ITestOptions
from funittest.core.events import TestStarting, TestEnding
from funittest.log import logger

class PolicyTestExecutor(SimpleTestExecutor):
    """Executor for policy tests.
    """

    implements(IPolicyTestExecutor)
    adapts(IPolicyTest, ITestOptions)

    def run(self, result):
        """Run a policy test.
        """
        for policytest_id, policytest in self.test.policytests.items():
            wrapped_result = PolicyTestResult(result, policytest_id, self.test)
            # The policytest is defined by a filename of the .csv file for the 
            # policy test configurations and the fixed preset
            filename = policytest['filename']
            fix = policytest['preset']
            logger.run_policytest(self.test, filename, fix)
            # Read the test settings from the file
            vary, scenario_names, suite = self.test.makeSuiteFromCSV(filename)
            objectEventNotify(TestStarting(self.test, policytest_id))
            # Now run the test for all the lines in the CSV file
            self.run_suite(wrapped_result, fix, vary, scenario_names, suite)
            objectEventNotify(TestEnding(self.test))
            wrapped_result.output(fix, vary, scenario_names)
        return result
    
    def run_suite(self, result, fix, vary, scenario_names, suite):
        """
        The configurations are given like this:
        
        fix: {'path': 'page_in_two_folders', 
              'typesetting': 'onestateworkflowfolder'}
        
        vary:  ['state', 'role']

        scenario_names: ['EditContent', 'ViewContent']

        suite:
        
        [('NoEditContent', {'role': 'anon',        'state': 'published'}),
         ('ViewContent',   {'role': 'anon',        'state': 'published'}),
         ('NoEditContent', {'role': 'contributor', 'state': 'published'}),
         ('ViewContent',   {'role': 'contributor', 'state': 'published'}), 
         ('EditContent',   {'role': 'editor',      'state': 'published'}),
         ('ViewContent',   {'role': 'editor',      'state': 'published'}),
         ('EditContent',   {'role': 'manager',     'state': 'published'}), 
         ('ViewContent',   {'role': 'manager',     'state': 'published'}), 
         ('NoEditContent', {'role': 'member',      'state': 'published'}),
         ('ViewContent',   {'role': 'member',      'state': 'published'}),
         ('EditContent',   {'role': 'owner',       'state': 'published'}), 
         ('ViewContent',   {'role': 'owner',       'state': 'published'}), 
         ('NoEditContent', {'role': 'reader',      'state': 'published'}),
         ('ViewContent',   {'role': 'reader',      'state': 'published'}),
         ('NoEditContent', {'role': 'reviewer',    'state': 'published'}),
         ('ViewContent',   {'role': 'reviewer',    'state': 'published'})]
        
        'ViewContent' is the scenario to be run
        'role' is the user that does the action
        'state' is the workflow state of the content of the path
        """
        self.start_test(self.test)
        self.set_fixed_on_test(fix)
        # Run suite by running all scenarios with all the prepared variations
        for scenario, vary in suite:
            # Prepare the test variables
            self.set_vary_on_test(vary)
            self.run_scenario(scenario, result, vary, scenario_names)
            self.set_fix_unchanged_on_test(fix)
        self.clean_test(vary, fix)
        self.end_test()

    def run_scenario(self, scenario, result, vary, scenario_names):
        """
        scenario: 'NoEditContent'
        """
        # The scenario should come from the same product as the test
        # For example, the plonepolicy tests have their scenarios in 
        # the plonepolicy product as well
        product = self.test.__parent__.get_name() 
        self.test.scenario = scenarios[product][scenario]
        result.before_execute()
        self.execute(result.unwrap_result())
        result.after_execute(vary, scenario, scenario_names)


    def clean_test(self, vary, fix):
        # Clean up the test object when the tests have run
        # FIXME: Search for a better way.
        if vary:
            for key in vary:
                if hasattr(self.test, '_' + key):
                    delattr(self.test, '_' + key)
                if hasattr(self.test, '_' + key + '_changed'):
                    delattr(self.test, '_' + key + '_changed')
                if hasattr(self.test, '_' + key + '_new'):
                    delattr(self.test, '_' + key + '_new')
        for key in fix.keys():
            if hasattr(self.test, '_' + key):
                delattr(self.test, '_' + key)
            if hasattr(self.test, '_' + key + '_changed'):
                delattr(self.test, '_' + key + '_changed')

    def set_fixed_on_test(self, fix):
        # Set attributes for all fixed items of the test schema
        for dp_name, dp_key in fix.items():
            if self.test.fix.has_key(dp_name):
                # FIXME: dp_key is a list and should be an item. Do we
                # want to run all over items of the list ? (and make a
                # cartesien product between all fixed data)
                value = self.test.fix[dp_name][dp_key[0]]
                
                # Set attribute using the dataprovider key
                setattr(self.test, '_' + dp_name, value)
                # We need a flag in order to do setup work only once
                setattr(self.test, '_' + dp_name + '_changed', True)
            else:
                raise IncompatibleSchema(self.test.fix, dp_name)              

    def set_vary_on_test(self, vary):
        """When running a test, the variables of the schema have to be
        set.  If the variables change, this has to be noted.  If this
        is the first time a variable is set, this has to be noted.
        """
        # Set vary values
        for dp_name, dp_key in vary.items():
            if self.test.vary.has_key(dp_name):
                # Retrieve value in dataprovider of the test
                value = self.test.vary[dp_name][dp_key]
                # Change value if different than the old one, and set
                # the flag when it has changed
                old = getattr(self.test, '_' + dp_name, None)
                if old != value:
                    setattr(self.test, '_' + dp_name, value)
                    setattr(self.test, '_' + dp_name + '_changed', True)
                else:
                    setattr(self.test, '_' + dp_name + '_changed', False)
                # Set flag when there was no setting before
                setattr(self.test, '_' + dp_name + '_new', old is None)
            else:
                raise IncompatibleSchema(self.test.vary, dp_name)

    def set_fix_unchanged_on_test(self, fix):
        # When the fixed values have been handled once in a scenario, 
        # they need to be flagged as unchanged, so the work is not done
        # any more in the following scenarios
        for key in fix.keys():
            setattr(self.test, '_' + key + '_changed', False)

provideAdapter(PolicyTestExecutor)
