import os
import random
import traceback

from types import DictType
from zope.interface import implements

from funittest.core.interfaces import IScenario
from funittest.problems import ScenarioExtensionNotFound
from funittest.problems import IncompatibleSchema
from funittest.problems import ConflictingSchemaConfiguration
from funittest.problems import FailedScenarioConfiguration
from funittest.problems import EnforceKeywordArguments
from funittest.problems import MissingStep
from funittest.problems import WrongSchema
from funittest.problems import ValueNotDict
from funittest.problems import NotPartOfSchema
from funittest.problems import ConfigurationError
from funittest.runner.baserunner import BaseTest
from funittest.runner.baserunner import TestInterpreterMixin
from funittest.core.assertions import TestCaseAssertions
from funittest.utils import product
from funittest.utils import FormatExceptionInfo
from funittest.utils import normalizeClassName
from funittest.log import logger

from funittest import interpreter
from funittest import scenarios
from funittest import dataprovider
from funittest.problems import ExpectedStateVerificationError
from funittest.problems import OnMethodExpectedStateError

class ScenarioConfigurationRunner(TestInterpreterMixin):
    """
    Evaluate the result of running a scenario on a site for different
    configurations.
    
    Either the scenario works under a configuration, or not.
    
    There can be different points at which the scenario stops working.
    
    The statistics of which scenario works or fails at which point is
    returned.
    """
    def __init__(self, scenario, extension=None, vary=[], fix={}, user=None):
        """
        scenario    - The scenario to be run, for example "register". If
                      no extension is given, the main scenario is run.
        extension   - The extension to run, for example "3a"
                      "all" means running the the main scenario and all
                      extensions
        vary        - Choose configurations by using all variations of 
                      parameters. Example: ["user", "content"] will vary keys
                      for users and content of the Schema.
        fix         - Fix configuration parameter. 
                      Example: {'user':'admin', 'content':'document1'}
                      Will make a configuration for only the admin user and
                      test with document1 only.
        """
        if extension:
            if not extension in scenario.list_extensions() and extension!='all':
                raise ScenarioExtensionNotFound(scenario, extension)
        if fix:
            # Make sure that the Schema has the fixed key
            for key in fix.keys():
                if not key in scenario.schema.keys():
                    raise IncompatibleSchema(scenario.schema, key)
        if vary:
            # Make sure that the Schema has the varied key
            for key in vary:
                if not key in scenario.schema.keys():
                    raise IncompatibleSchema(scenario.schema, key)
        
        if fix and vary:
            # Make sure that what is fixed is not varied
            for key in vary:
                if key in fix.keys():
                    raise ConflictingSchemaConfiguration(key)

        self.scenario = scenario
        self.extension = extension
        self.vary = vary
        self.fix = fix
        self.user = user

    def get_configurations(self):
        """
        Get configurations for the scenario
        
        1. If only the scenario is given, run the main extension
        
        2. If a specific extension is given, run that
        
        3. If the extension "all" is given, run the main scenario and all extensions
        """
        configurations = []
        if self.extension == 'all':
            # Test with the main extension and all other extensions
            extensions = [None,] + self.scenario.list_extensions()
        else:
            # Test with the given extension
            extensions = [self.extension,]

        for extension in extensions:
            if self.vary or self.fix:
                for configuration in self.scenario.get_selected_configurations(self.vary, self.fix):
                    configurations.append( (extension, configuration) )
            else:
                configurations.append( (extension, self.scenario.get_any_configuration()) )

        return configurations

    def run_configurations(self, configurations):
        """
        Return configurations that ran ok and not ok
        """
        logger.run_scenario(self.scenario)
        ok = []
        ko = []
        for extension, configuration in configurations:
            logger.run_scenario_extension(extension, configuration)
            if self.run_configuration(configuration, extension):
                ok.append(configuration)
            else:
                ko.append(configuration)
        return ok, ko

    def run_configuration(self, configuration, extension=None):
        self.scenario.configure(configuration, extension)
        self.start_test(self.scenario)

        if self.user:
            # FIXME This is plone specific
            user = dataprovider.plone.user.get(self.user)
            scenarios.plone.login(user=user)

        was_successful = True

        try:
            self.scenario.run()
        except (ExpectedStateVerificationError, OnMethodExpectedStateError, AssertionError):
            # No exception, error can happen
            was_successful = False
        except:
            logger.exception()
            raise

        interpreter.pause_for_debugging(was_successful)

        self.end_test()

        return was_successful

class Scenario(BaseTest, TestCaseAssertions):
    """
    Base class for scenarios
    
    A scenario can be executed with different configurations.
    """
    implements(IScenario)
    
    def __call__(self, *args, **kw):
        if args:
            raise EnforceKeywordArguments(args)
        self.set_schema(args, kw)
        logger.start_scenario(self)
        self.scenario()
        logger.end_scenario()

    def set_schema(self, args, kw):
        """
        Set the schema attributes. For example, the kw
        
        kw = {'transition': {'id': 'publish'}}
        
        becomes an attribute of the scenario instance:
        
        self._transition = {'id': 'publish'}
        """
        for key, value in kw.items():
            if not self.schema.has_key(key):
                raise NotPartOfSchema(self.get_name(), self.schema.keys(), key)
            for k in self.schema[key].defaults.keys():
                # Go through all of default keys of each the dataprovider
                if type(value) is DictType:
                    if not k in value.keys():
                        # All of the default keys of the dataprovider must be present
                        raise WrongSchema(schema=self.schema[key], value=value)
                else:
                    # The Schema values are enforced to be dictionaries
                    raise ValueNotDict(key=key, value=value)
            # For each schema key, set the value
            setattr(self, '_'+key, value)

    def is_schema_set(self):
        """
        Verify that all schema keys are available as attributes
        """
        is_set = True
        for key in self.schema.keys():
            if not hasattr(self, '_'+key):
                # The attribute for this schema key is not set
                is_set = False
            else:
                # Let's make sure the value of the attribute is a dictionary
                value = getattr(self, '_'+key, None)
                if not type(value) is DictType:
                    raise ValueNotDict(key=key, value=value)
        return is_set

    def precondition(self):
        return True

    def get_any_configuration(self):
        """
        For all dataproviders in the schema, choose a data key for
        the configuration at random. In case the current attribute is
        set, take the keys from that, as it indicates what it currently
        available.
        """
        configuration = {}
        for key in self.schema.keys():
            if type(self.schema[key].current)==type(""):
                # The current is given as a string, so put it in a list
                # so that we can choose from it later, keeping things simple
                keys = [self.schema[key].current]
            else:
                if self.schema[key].current:
                    # The currently available keys are set, so we take expect
                    # these keys to be available, and choose from them
                    keys = self.schema[key].current
                else:
                    # The currently available keys are not fixed, so we have
                    # no further indication what is available or not, so let's
                    # just expect we can freely choose
                    keys = self.schema[key].keys()
            if len(keys)==0:
                # Nothing to choose from, so take None
                configuration[key]=None
            else:
                # Choose one of the keys
                configuration[key]=random.choice(keys)
        return configuration

    def get_selected_configurations(self, vary, fix, current=False):
        sequence = []
        for key in self.schema.keys():
            if current and self.schema[key].current:
                keys = self.schema[key].current
            else:
                keys = self.schema[key].keys()
            if key in vary:
                # Variations on this key
                random.shuffle(keys)
                sequence.append(keys)
            elif fix.has_key(key):
                # Fixed choice
                sequence.append([fix[key]])
            else:
                # Only one choice at random
                if len(keys):
                    sequence.append([random.choice(keys),])
                else:
                    raise ConfigurationError(key=key)
        sequences = product(*sequence)
        configurations = []
        for seq in sequences:
            configuration={}
            i=0
            for key in self.schema.keys():
                configuration[key]=seq[i]
                i+=1
            configurations.append(configuration)
        return configurations
    
    def configure(self, configuration, extension=None):
        for key in self.schema.keys():
            configuration_key = configuration[key]
            schema = self.schema[key]
            if schema.has_key(configuration_key):
                setattr(self, '_'+key, schema.get(configuration_key))
            else:
                logger.dataprovider_key_not_found(schema, configuration_key)
                raise FailedScenarioConfiguration(self, schema, configuration_key)
        self.extension = extension
        self.configuration = configuration

    def get_configuration(self):
        if hasattr(self, 'configuration'):
            return self.configuration
        else:
            return None

    def run(self):
        if hasattr(self, 'setUp'):
            self.setUp()
        if self.extension:
            getattr(self, 'scenario_' + self.extension)()
        else:
            self.scenario()

    def list_extensions(self):
        extensions = []
        for m in dir(self):
            if m.startswith('scenario_'):
                extensions.append(m[len('scenario_'):])
        extensions.sort()
        return extensions
    
    def expect_ok(self, *steps):
        assert(self.is_schema_set())
        for step in steps:
            getattr(self, 'step_' + str(step))()

    def try_step(self, step):
        assert(self.is_schema_set())
        ok = True
        step_method = getattr(self, 'step_' + str(step), None)
        if not step_method:
            raise MissingStep(self, step)
        else:
            try:
                step_method()
            except (ExpectedStateVerificationError, AssertionError):
                # No exception, error expected
                ok=False
            except:
                logger.exception()
                raise
        return ok

    def try_scenario(self, *args, **kw):
        if args:
            raise EnforceKeywordArguments(args)
        self.set_schema(args, kw)
        assert(self.is_schema_set())
        ok = True
        try:
            self.scenario()
        except (ExpectedStateVerificationError, AssertionError):
            # No exception, error expected
            ok=False
        except:
            logger.exception()
            raise
        return ok

