# -*- coding: utf-8 -*-
## funittest
## Functional test automation stack
## Copyright (C)2007 Ingeniweb

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING. If not, write to the
## Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import signal
from xml.sax.saxutils import quoteattr
from pkg_resources import working_set
from pkg_resources import Requirement
from pkg_resources import resource_filename
from pkg_resources import Environment

from zope.pagetemplate.pagetemplatefile import PageTemplateFile
from zope.pagetemplate.pagetemplate import PTRuntimeError
from zope.tal.talinterpreter import TALInterpreter

import imp, traceback
import time, os, os.path, sys, codecs

def normalizeName(klass):
    """Simply return the name of a object in lower case.
    """
    return klass.__name__.lower()


def normalizeClassName(klass):
    """Simply return the class name in lower case.
    """
    return klass.__class__.__name__.lower()


def _funittestExtensions():
    """Return all funittest extensions, which can be recognize
       by their prefix: "funittest."
       
       We are only interested in eggs starting with 'funittest.', for
       example 'funittest.plone' or 'funittest.silva'.

       Plugging in your own extensions is rather easy due to this system.
    """
    extensions = []
    # Go through all modules and look for extensions to funittest
    for egg in working_set:
        # The egg.key can be for example 'zc.testbrowser', or 'elementtree'
        if egg.key.startswith('funittest.'):
            extensions.append(egg)
    return extensions

def rescanEggs():
    """Rescan for new eggs in the environment.
    """
    loader_env = Environment()
    loader_env.scan(sys.path)
    eggs, _ = working_set.find_plugins(loader_env)
    map(working_set.add, eggs)

def retrieveVersion(short=False):
    """Retrieve funittest, and funittest extension version.
    """

    funittest = working_set.find(Requirement.parse('funittest'))
    version = '%s %s' % (funittest.project_name, funittest.version)

    if not short:
        for egg in _funittestExtensions():
            version += '\n%s %s' % (egg.project_name, egg.version)
    
    return version


def listAvailablePythonModules(python_path):
    """List availables modules in the python_path, going through all
       eggs for funittest.
    """
    modules = dict()
    INIT_MODULES = ['__init__.py', '__init__.pyc']
    path_component = python_path.split('.')

    for egg in _funittestExtensions():
        path = os.path.join(egg.location, *path_component)
        try:
            for possible in os.listdir(path):
                if possible in INIT_MODULES: # We don't want init files.
                    continue

                possible_path = os.path.join(path, possible)
                if os.path.isdir(possible_path):
                    possible_contents = os.listdir(possible_path)
                    for init in INIT_MODULES:
                        if init in possible_contents:
                            modules[possible] = '%s.%s' % (python_path, possible)
                            break
                else:
                    for suffix, type, mod in imp.get_suffixes():
                        if possible.endswith(suffix):
                            name = possible[:-len(suffix)]
                            modules[name] = '%s.%s' % (python_path, name)
                            break
        except OSError:
            pass
    return modules


def resourceFile(name, egg='funittest'):
    """Return path to the given resource file.
       
       This is used in some dataproviders that provider file or image content.
       Example files are searched in the resources folder of the funittest 
       extension module, containing the dataprovider.
       
       Example use in the dp_content.py dataprovider:
       
           resourceFile("plone_wheel.png", egg='funittest.plone')
    """
    req = Requirement.parse(egg)
    return resource_filename(req, os.path.join("funittest", "resources", name))

class FileAsPageTemplate(PageTemplateFile):
    """PageTemplate writing its output in an UTF-8 file.
    """

    def pt_getContext(self, args=(), options={}):
        """Set context correctly to access to funittest registery.
        """

        ctxt = super(FileAsPageTemplate, self).pt_getContext(args)
        ctxt.update(options)
        if not 'resources' in ctxt: # Include default funittest CSS
            ctxt['resources'] = dict(css=resourceFile('funittest.css'))
        ctxt['time'] = time.asctime()
        ctxt['version'] = retrieveVersion()
        self.output_name = options.get('filename', 'default.html')
        return ctxt

    def pt_render(self, namespace, source=False, sourceAnnotations=False,
                  showtal=False):
        """Render this Page Template, writing the output in a file.
        """
        output = codecs.open(self.output_name, 'w', 'utf-8')
        rendered = super(PageTemplateFile, self).pt_render(namespace, source=False, sourceAnnotations=False, showtal=False)
        output.write(rendered)
        output.close()

# Taken from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/302478
def product(*seqin):
    '''
    returns a generator which returns combinations of argument sequences
    for example xcombine((1,2),(3,4)) returns a generator; calling the next()
    method on the generator will return [1,3], [1,4], [2,3], [2,4] and
    StopIteration exception.  This will not create the whole list of
    combinations in memory at once.
    '''

    def rloop(seqin,comb):
        '''recursive looping function'''
        if seqin:                   # any more sequences to process?
            for item in seqin[0]:
                newcomb=comb+[item]     # add next item to current combination
                # call rloop w/ remaining seqs, newcomb
                for item in rloop(seqin[1:],newcomb):
                    yield item          # seqs and newcomb
        else:                           # processing last sequence
            yield comb                  # comb finished, add to list
    return rloop(seqin,[])

def FormatExceptionInfo(quote = False):
    exc_info = sys.exc_info()
    format_exception = traceback.format_exception(*exc_info)
    if quote:
        return quoteattr(' '.join(format_exception))
    else:
        return ' '.join(format_exception)
    

def get_zope_methods():
    makerequest = None
    newSecurityManager = None
    try:
        from Testing.ZopeTestCase.utils import makerequest
    except:
        pass
    try:
        from AccessControl.SecurityManagement import newSecurityManager
    except:
        pass
    return makerequest, newSecurityManager

class ZopeApp:

    def __init__(self):
        frame = sys._getframe(2)
        self.app = frame.f_locals.get('app', None)
        makerequest, newSecurityManager = self.get_zope_methods()
        self.makerequest = makerequest
        self.newSecurityManager = newSecurityManager

    def get_zope_methods(self):
        return get_zope_methods()
    
    def initSession(self, username):
        """Init a Zope Session as username.
        """
        zope_app_session = self.makerequest(self.app)
        user = zope_app_session.acl_users.getUserById(username)
        self.newSecurityManager(None, user.__of__(zope_app_session.acl_users))
        return zope_app_session
    
    def commit_transaction(self):
        import transaction
        transaction.commit()

    def closeSession(self, zope_app_session):
        """Commit changes and close the ZOBD.
        """
        app = zope_app_session.getPhysicalRoot()
        app._p_jar.close()

class SubProcess(object):
    def __init__(self):
        self.pids=[]

    def _prepareArgs(self, command, args):
        real = [command]
        real.extend(args)
        return real
        
    def start(self, log_name, command, args, env, error_callback, verbose):
        pid = os.fork()
        # os.fork:
        # Return 0 to child process and PID of child to parent process.
        if not pid:
            log = os.open(log_name, os.O_RDWR | os.O_CREAT | os.O_APPEND, 0644)
            # os.getpid:
            # Return the current process id
            pid = os.getpid()
            # os.setpgid:
            # Call the system call setpgid(), which sets the process group of
            # the specified process pid to the specified pgrp.
            os.setpgid(pid, pid)
            # os.dup2:
            # Duplicate file descriptor.
            os.dup2(log, 1)
            os.dup2(log, 2)
            os.close(log)
            try:
                # os.execvpe:
                # Execute the executable file (which is searched for along $PATH)
                # with argument list args and environment env , replacing the
                # current process.
                os.execvpe(command, self._prepareArgs(command, args), env)
            except OSError:
                error_callback("Command not found: %s" % command)
        self.pids.append(pid)
        if verbose:
            print "Running command %s with args %s and PID %s" % (command, args, pid)
    
    def stop_all(self):
        for pid in self.pids:
            try:
                os.kill(-pid, signal.SIGTERM)
            except OSError:
                pass

def set_testcase(interpreter, testcase):
    # Need a central place to store a reference to the testcase
    interpreter.wrappedinterpreter._testcase=testcase

def press_key_to_continue(interpreter, prompt):
    if not interpreter.is_dry_run():
        raw_input(prompt)

