""" preferences.py:
    Keeps track of user preferences.
    In future we should provides a GUI for changing the preferences.
"""

import os
from ConfigParser import SafeConfigParser

class Singleton(object):
    """From the 2nd edition of the Python cookbook.
       Ensures that only one instance is created per running script, so
       that it can be accessed from any running module as a unique object."""
    def __new__(cls, *args, **kwargs):
        if '_inst' not in vars(cls):
            cls._inst = object.__new__(cls, *args, **kwargs)
        return cls._inst

class UserPreferences(Singleton):
    """Keeps track of user preferences such as:
    language: the preferred language;
    working_dir: the directory where user-created Python files are saved;
    style: the preferred css style.
    Eventually, more options (e.g. styling of Python programs) will be added.
    """
    def __init__(self):
        self.config = SafeConfigParser()
        self.changed = False
        self.user_dir = os.path.join(os.path.expanduser("~"), ".crunchy")
        if not os.path.exists(self.user_dir):  # first time
            self.create_path()
        self.user_file = os.path.join(self.user_dir, "crunchy.cfg")
        self.load()
        return
    
    def create_path(self):
        '''Creates a path for Crunchy when none exists.'''
        try:
            os.makedirs(self.user_dir)
        except:
            print "Could not create the user directory."
            self.user_dir = os.getcwd()  # use crunchy's a a default.
        return
    
    def create_file(self):
        '''Creates a configuration file when none exist.'''
        self.config.add_section('preferences')
        self.set_language('en')
        self.set_working_dir(self.user_dir)
        self.set_style('gradient1.css')
        f = open(self.user_file, 'w')
        self.config.write(f)
        f.close()
        return
    
    def set_preference(self, option, value):
        '''Set an individual preference option'''
        self.config.set('preferences', option, value)
        self.changed = True
        return
    
    def load(self):
        '''Reads the configuration values'''
        try:
            self.config.readfp(open(self.user_file, 'r'))
        except:
            self.create_file()
        self.working_dir = self.config.get('preferences', 'working directory')
        self.style = self.config.get('preferences', 'style')
        self.language = self.config.get('preferences', 'language')
        self.changed = False
        return
    
    def save(self):
        '''If necessary, saves the changed values.'''
        if self.changed:
            f = open(self.user_file, 'w')
            self.config.write(f)
            f.close()
            self.changed = False
        return
    
    #== the following are implemented as properties for transparent access

    def get_working_dir(self):
        return self._working_dir

    def set_working_dir(self, path):
        self._working_dir = path
        self.set_preference('working directory', path)
        return

    def get_language(self):
        return self._language

    def set_language(self, lang):
        self._language = lang
        self.set_preference('language', lang)
        return

    def get_style(self):
        return self._style

    def set_style(self, style):
        self._style = style
        self.set_preference('style', style)
        return
    
    working_dir = property(get_working_dir, set_working_dir)
    language = property(get_language, set_language)
    style = property(get_style, set_style)

